# TestInOutVariables.py This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that @inout variables display reasonably
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestInOutVariables(TestBase):
    @swiftTest
    def test_in_out_variables(self):
        """Test that @inout variables display reasonably"""
        self.build()
        self.do_test()

    def setUp(self):
        TestBase.setUp(self)
        self.main_source = "main.swift"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    def check_class_internal(
            self,
            ivar_value,
            ovar_value,
            use_expression,
            is_wrapped):
        """is_wrapped allows unwrapping of an inout type.
           These are currently not generated by the compiler"""
        if use_expression:
            # At present the expression parser strips off the @lvalue bit:
            message_end = "from EvaluateExpression"
            x_actual = self.frame().EvaluateExpression(
                "x", lldb.eDynamicCanRunTarget)
            self.assertSuccess(
                x_actual.GetError(),
                "Expression evaluation failed")
        else:
            message_end = "from FindVariable"
            x = self.frame().FindVariable("x").GetDynamicValue(
                lldb.eDynamicCanRunTarget)
            self.assertTrue(x.IsValid(), "did not find x %s" % (message_end))
            if is_wrapped:
                self.assertTrue(
                    x.GetNumChildren() == 1,
                    "x has too many children %s" %
                    (message_end))
                x_actual = x.GetChildAtIndex(0).GetDynamicValue(
                    lldb.eDynamicCanRunTarget)
                self.assertTrue(
                    x_actual.IsValid(),
                    "did not find the child of x %s" %
                    (message_end))
            else:
                x_actual = x

        ivar = x_actual.GetChildAtIndex(0).GetChildAtIndex(0)
        ovar = x_actual.GetChildAtIndex(1)
        self.assertEqual(
            ivar.GetName(), "ivar", "Name: %s is not ivar %s" %
            (ivar.GetName(), message_end))
        self.assertEqual(
            ovar.GetName(), "ovar",
            "ovar is not ovar %s" %
            (message_end))
        self.assertEqual(
            ivar.GetValue(), ivar_value,
            "ivar wrong %s" %
            (message_end))
        self.assertEqual(
            ovar.GetValue(), ovar_value,
            "ovar wrong %s" %
            (message_end))

    def check_class(self, ivar_value, ovar_value, is_wrapped=False):
        self.check_class_internal(ivar_value, ovar_value, True, is_wrapped)
        self.check_class_internal(ivar_value, ovar_value, False, is_wrapped)

    def check_struct_internal(self, ivar_value, use_expression):
        if use_expression:
            x = self.frame().EvaluateExpression("x", lldb.eDynamicCanRunTarget)
            message_end = "from EvaluateExpression"
        else:
            x = self.frame().FindVariable("x").GetDynamicValue(
                lldb.eDynamicCanRunTarget)
            message_end = "from FindVariable"

            # Make sure we can get the object description without crashing
            _ = self.frame().FindVariable("x").description

        self.assertTrue(x.IsValid(), "did not find x %s" % (message_end))
        self.assertTrue(
            x.GetNumChildren() == 1,
            "x has too many children %s" %
            (message_end))
        ivar = x.GetChildAtIndex(0)
        if not "unwrap":
            if not use_expression:
                ivar = ivar.GetChildAtIndex(0)
        self.assertTrue(
            ivar.GetName() == "ivar",
            "ivar is not ivar %s" %
            (message_end))
        self.assertTrue(
            ivar.GetValue() == ivar_value,
            "ivar wrong %s" %
            (message_end))

    def check_struct(self, ivar_value):
        self.check_struct_internal(ivar_value, True)
        self.check_struct_internal(ivar_value, False)

    def check_next_stop(self, breakpoint):
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            self.process, breakpoint)
        self.assertTrue(len(threads) == 1)

    def do_test(self):
        """Test that @inout variables display reasonably"""
        exe_name = "a.out"
        exe = self.getBuildArtifact(exe_name)

        # Create the target
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints
        class_bkpt = target.BreakpointCreateBySourceRegex(
            'Set breakpoint here for Class access', self.main_source_spec)
        self.assertTrue(class_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        struct_bkpt = target.BreakpointCreateBySourceRegex(
            'Set breakpoint here for Struct access', self.main_source_spec)
        self.assertTrue(struct_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        outer_bkpt = target.BreakpointCreateBySourceRegex(
            'Set breakpoint here for String access', self.main_source_spec)
        self.assertTrue(outer_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        fn_ptr_bkpt = target.BreakpointCreateBySourceRegex(
            'Set breakpoint here for Function type with inout', self.main_source_spec)
        self.assertTrue(fn_ptr_bkpt.GetNumLocations() > 0, VALID_BREAKPOINT)

        # Launch the process, and do not stop at the entry point.
        self.process = target.LaunchSimple(None, None, os.getcwd())

        self.assertTrue(self.process, PROCESS_IS_VALID)

        # Frame #0 should be at our breakpoint.
        self.check_next_stop(class_bkpt)
        self.check_class("4321", "112233")

        self.process.Continue()
        self.check_next_stop(struct_bkpt)
        self.check_struct("4567")

        self.process.Continue()
        self.check_next_stop(class_bkpt)
        self.check_class("4322", "112233")

        # Now on the way out let's modify the class and make sure it gets
        # modified both here and outside the functions:
        var = self.frame().EvaluateExpression(
            "x = Other(in1: 556678, in2: 667788)",
            lldb.eDynamicCanRunTarget)
        self.assertSuccess(var.GetError())

        #self.check_class("556677", "667788")

        self.process.Continue()
        self.check_next_stop(outer_bkpt)

        svar = self.frame().FindVariable("x")
        if not "unwrap":
            svar = svar.GetChildAtIndex(0)
        self.assertTrue(svar.GetSummary() == '"Keep going, nothing to see"')

        self.process.Continue()
        self.check_next_stop(class_bkpt)
        self.check_class("556679", "667788")

        self.process.Continue()
        self.check_next_stop(struct_bkpt)
        self.check_struct("4568")

        # Now we want to run to the fn_ptrs breakpoint.
        struct_bkpt.SetEnabled(False)
        class_bkpt.SetEnabled(False)
        outer_bkpt.SetEnabled(False)
        
        self.process.Continue()
        self.check_next_stop(fn_ptr_bkpt)
        dict_var = self.frame().FindVariable("dict").GetChildAtIndex(0)
        self.assertTrue(dict_var.IsValid(), "Got an invalid value for the first element: %s"%(dict_var.GetError().GetCString()))
        func_value = dict_var.GetChildMemberWithName("value")
        self.assertTrue(func_value.IsValid(), "Got an invalid value for func ptr: %s"%(func_value.GetError().GetCString()))
        func_sc = target.ResolveSymbolContextForAddress(target.ResolveLoadAddress(func_value.GetValueAsUnsigned()), lldb.eSymbolContextFunction)
        self.assertTrue(func_sc.GetFunction().IsValid(), "We couldn't look up the function target.") 




"""
Test saving a mini dump.
"""


import os
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ProcessSaveCoreMinidumpTestCase(TestBase):
    @skipUnlessArch("x86_64")
    @skipUnlessPlatform(["linux"])
    def test_save_linux_mini_dump(self):
        """Test that we can save a Linux mini dump."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        core = self.getBuildArtifact("core.dmp")
        core_sb = self.getBuildArtifact("core_sb.dmp")
        try:
            target = self.dbg.CreateTarget(exe)
            process = target.LaunchSimple(
                None, None, self.get_process_working_directory()
            )
            self.assertState(process.GetState(), lldb.eStateStopped)

            # get neccessary data for the verification phase
            process_info = process.GetProcessInfo()
            expected_pid = process_info.GetProcessID() if process_info.IsValid() else -1
            expected_number_of_modules = target.GetNumModules()
            expected_modules = target.modules
            expected_number_of_threads = process.GetNumThreads()
            expected_threads = []

            for thread_idx in range(process.GetNumThreads()):
                thread = process.GetThreadAtIndex(thread_idx)
                thread_id = thread.GetThreadID()
                expected_threads.append(thread_id)

            # save core and, kill process and verify corefile existence
            self.runCmd(
                "process save-core --plugin-name=minidump --style=stack " + core
            )
            self.assertTrue(os.path.isfile(core))

            # validate savinig via SBProcess
            error = process.SaveCore(core_sb, "minidump", lldb.eSaveCoreStackOnly)
            self.assertTrue(error.Success())
            self.assertTrue(os.path.isfile(core_sb))

            error = process.SaveCore(core_sb, "minidump", lldb.eSaveCoreFull)
            self.assertTrue(error.Fail())
            error = process.SaveCore(core_sb, "minidump", lldb.eSaveCoreDirtyOnly)
            self.assertTrue(error.Fail())

            self.assertSuccess(process.Kill())

            # To verify, we'll launch with the mini dump
            target = self.dbg.CreateTarget(None)
            process = target.LoadCore(core)

            # check if the core is in desired state
            self.assertTrue(process, PROCESS_IS_VALID)
            self.assertTrue(process.GetProcessInfo().IsValid())
            self.assertEqual(process.GetProcessInfo().GetProcessID(), expected_pid)
            self.assertTrue(target.GetTriple().find("linux") != -1)
            self.assertTrue(target.GetNumModules(), expected_number_of_modules)
            self.assertEqual(process.GetNumThreads(), expected_number_of_threads)

            for module, expected in zip(target.modules, expected_modules):
                self.assertTrue(module.IsValid())
                module_file_name = module.GetFileSpec().GetFilename()
                expected_file_name = expected.GetFileSpec().GetFilename()
                # skip kernel virtual dynamic shared objects
                if "vdso" in expected_file_name:
                    continue
                self.assertEqual(module_file_name, expected_file_name)
                self.assertEqual(module.GetUUIDString(), expected.GetUUIDString())

            for thread_idx in range(process.GetNumThreads()):
                thread = process.GetThreadAtIndex(thread_idx)
                self.assertTrue(thread.IsValid())
                thread_id = thread.GetThreadID()
                self.assertTrue(thread_id in expected_threads)
        finally:
            # Clean up the mini dump file.
            self.assertTrue(self.dbg.DeleteTarget(target))
            if os.path.isfile(core):
                os.unlink(core)
            if os.path.isfile(core_sb):
                os.unlink(core_sb)



"""Show bitfields and check that they display correctly."""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CppBitfieldsTestCase(TestBase):
    @no_debug_info_test
    def test_bitfields(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        # Accessing LargeBitsA.
        self.expect_expr(
            "lba",
            result_children=[
                ValueCheck(name="", type="int:32"),
                ValueCheck(name="a", type="unsigned int:20", value="2"),
            ],
        )
        self.expect_expr("lba.a", result_type="unsigned int", result_value="2")

        # Accessing LargeBitsB.
        self.expect_expr(
            "lbb",
            result_children=[
                ValueCheck(name="a", type="unsigned int:1", value="1"),
                ValueCheck(name="", type="int:31"),
                ValueCheck(name="b", type="unsigned int:20", value="3"),
            ],
        )
        self.expect_expr("lbb.b", result_type="unsigned int", result_value="3")

        # Accessing LargeBitsC.
        self.expect_expr(
            "lbc",
            result_children=[
                ValueCheck(name="", type="int:22"),
                ValueCheck(name="a", type="unsigned int:1", value="1"),
                ValueCheck(name="b", type="unsigned int:1", value="0"),
                ValueCheck(name="c", type="unsigned int:5", value="4"),
                ValueCheck(name="d", type="unsigned int:1", value="1"),
                ValueCheck(name="", type="int:2"),
                ValueCheck(name="e", type="unsigned int:20", value="20"),
            ],
        )
        self.expect_expr("lbc.c", result_type="unsigned int", result_value="4")

        # Accessing LargeBitsD.
        self.expect_expr(
            "lbd",
            result_children=[
                ValueCheck(name="arr", type="char[3]", summary='"ab"'),
                ValueCheck(name="", type="int:32"),
                ValueCheck(name="a", type="unsigned int:20", value="5"),
            ],
        )
        self.expect_expr("lbd.a", result_type="unsigned int", result_value="5")

        # Test BitfieldsInStructInUnion.
        # FIXME: This needs some more explanation for what it's actually testing.
        nested_struct_children = [
            ValueCheck(name="", type="int:22"),
            ValueCheck(name="a", type="uint64_t:1", value="1"),
            ValueCheck(name="b", type="uint64_t:1", value="0"),
            ValueCheck(name="c", type="uint64_t:1", value="1"),
            ValueCheck(name="d", type="uint64_t:1", value="0"),
            ValueCheck(name="e", type="uint64_t:1", value="1"),
            ValueCheck(name="f", type="uint64_t:1", value="0"),
            ValueCheck(name="g", type="uint64_t:1", value="1"),
            ValueCheck(name="h", type="uint64_t:1", value="0"),
            ValueCheck(name="i", type="uint64_t:1", value="1"),
            ValueCheck(name="j", type="uint64_t:1", value="0"),
            ValueCheck(name="k", type="uint64_t:1", value="1"),
        ]
        self.expect_expr(
            "bitfields_in_struct_in_union",
            result_type="BitfieldsInStructInUnion",
            result_children=[
                ValueCheck(
                    name="",
                    children=[ValueCheck(name="f", children=nested_struct_children)],
                )
            ],
        )
        self.expect_expr(
            "bitfields_in_struct_in_union.f.a", result_type="uint64_t", result_value="1"
        )

        # Unions with bitfields.
        self.expect_expr(
            "uwbf",
            result_type="UnionWithBitfields",
            result_children=[
                ValueCheck(name="a", value="255"),
                ValueCheck(name="b", value="65535"),
                ValueCheck(name="c", value="4294967295"),
                ValueCheck(name="x", value="4294967295"),
            ],
        )
        self.expect_expr(
            "uwubf",
            result_type="UnionWithUnnamedBitfield",
            result_children=[
                ValueCheck(name="a", value="16777215"),
                ValueCheck(name="x", value="4294967295"),
            ],
        )

        # Class with a base class and a bitfield.
        self.expect_expr(
            "derived",
            result_type="Derived",
            result_children=[
                ValueCheck(
                    name="Base",
                    children=[ValueCheck(name="b_a", value="2", type="uint32_t")],
                ),
                ValueCheck(name="d_a", value="1", type="uint32_t:1"),
            ],
        )

        # Struct with bool bitfields.
        self.expect_expr(
            "bb",
            result_type="",
            result_children=[
                ValueCheck(name="a", value="true", type="bool:1"),
                ValueCheck(name="b", value="false", type="bool:1"),
                ValueCheck(name="c", value="true", type="bool:2"),
                ValueCheck(name="d", value="true", type="bool:2"),
            ],
        )

        bb = self.frame().FindVariable("bb")
        self.assertSuccess(bb.GetError())

        bb_a = bb.GetChildAtIndex(0)
        self.assertSuccess(bb_a.GetError())
        self.assertEqual(bb_a.GetValueAsUnsigned(), 1)
        self.assertEqual(bb_a.GetValueAsSigned(), 1)

        bb_b = bb.GetChildAtIndex(1)
        self.assertSuccess(bb_b.GetError())
        self.assertEqual(bb_b.GetValueAsUnsigned(), 0)
        self.assertEqual(bb_b.GetValueAsSigned(), 0)

        bb_c = bb.GetChildAtIndex(2)
        self.assertSuccess(bb_c.GetError())
        self.assertEqual(bb_c.GetValueAsUnsigned(), 1)
        self.assertEqual(bb_c.GetValueAsSigned(), 1)

        bb_d = bb.GetChildAtIndex(3)
        self.assertSuccess(bb_d.GetError())
        self.assertEqual(bb_d.GetValueAsUnsigned(), 1)
        self.assertEqual(bb_d.GetValueAsSigned(), 1)

        # Test a class with a base class that has a vtable ptr. The derived
        # class has bitfields.
        base_with_vtable_children = [
            ValueCheck(name="a", type="unsigned int:4", value="5"),
            ValueCheck(name="b", type="unsigned int:4", value="0"),
            ValueCheck(name="c", type="unsigned int:4", value="5"),
        ]
        self.expect_expr("base_with_vtable", result_children=base_with_vtable_children)
        self.expect_var_path("base_with_vtable", children=base_with_vtable_children)

    @no_debug_info_test
    def test_bitfield_behind_vtable_ptr(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        # Test a class with a vtable ptr and bitfields.
        with_vtable_children = [
            ValueCheck(name="a", type="unsigned int:4", value="5"),
            ValueCheck(name="b", type="unsigned int:4", value="0"),
            ValueCheck(name="c", type="unsigned int:4", value="5"),
        ]
        self.expect_expr("with_vtable", result_children=with_vtable_children)
        self.expect_var_path("with_vtable", children=with_vtable_children)

        # Test a class with a vtable ptr and unnamed bitfield directly after.
        with_vtable_and_unnamed_children = [
            ValueCheck(name="", type="int:4", value="0"),
            ValueCheck(name="b", type="unsigned int:4", value="0"),
            ValueCheck(name="c", type="unsigned int:4", value="5"),
        ]
        self.expect_expr(
            "with_vtable_and_unnamed", result_children=with_vtable_and_unnamed_children
        )
        self.expect_var_path(
            "with_vtable_and_unnamed", children=with_vtable_and_unnamed_children
        )

        derived_with_vtable_children = [
            ValueCheck(
                name="Base",
                children=[ValueCheck(name="b_a", value="2", type="uint32_t")],
            ),
            ValueCheck(name="a", value="1", type="unsigned int:1"),
        ]
        self.expect_expr(
            "derived_with_vtable", result_children=derived_with_vtable_children
        )
        self.expect_var_path(
            "derived_with_vtable", children=derived_with_vtable_children
        )



#!/usr/bin/env python

import argparse
import re
import sys


def main():
    argp = argparse.ArgumentParser()
    argp.add_argument("infile", type=argparse.FileType("rb"), help="Input vmcore file")
    argp.add_argument(
        "outfile", type=argparse.FileType("wb"), help="Output vmcore file"
    )
    args = argp.parse_args()

    inf = args.infile
    outf = args.outfile
    line_re = re.compile(r"^% RD: (\d+) (\d+)")

    # copy the first chunk that usually includes ELF headers
    # (not output by patched libfbsdvmcore since libelf reads this)
    outf.write(inf.read(1024))

    for l in sys.stdin:
        m = line_re.match(l)
        if m is None:
            continue
        offset, size = [int(x) for x in m.groups()]

        inf.seek(offset)
        outf.seek(offset)
        outf.write(inf.read(size))


if __name__ == "__main__":
    main()



def foo2_function(debugger, args, result, dict):
    print("foo2 says " + args, file=result)
    return None


def __lldb_init_module(debugger, session_dict):
    debugger.HandleCommand("command script add -f foo2.foo2_function foo2cmd")
    return None



import lldb
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbtest as lldbtest
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftLazyFramework(lldbtest.TestBase):

    NO_DEBUG_INFO_TESTCASE = True
    mydir = lldbtest.TestBase.compute_mydir(__file__)

    @swiftTest
    @skipIf(oslist=no_match(["macosx"]))
    @skipIf(setting=('symbols.swift-precise-compiler-invocation', 'true'))
    def test_system_framework(self):
        """Test that a framework that is registered as autolinked in a Swift
           module used in the target, but not linked against the target is
           automatically loaded by LLDB."""
        self.build()
        self.expect("settings set target.swift-auto-import-frameworks true")
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))

        # Verify that lazy is not linked in.
        self.runCmd("image list")
        output = self.res.GetOutput()
        self.assertIn("dyld", output)
        self.assertNotIn("Lazy.framework/Versions/A/Lazy", output)
        # FIXME: we should automatically retry the expression on dylib import.
        self.expect("expression -- 1", error=True)
        self.expect("expression -- C()", substrs=['23'])

        # Verify that lazy has been dynamically loaded.
        self.expect("image list", substrs=["Lazy.framework/Versions/A/Lazy"])



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCaseClassTemplateTypeParameterPack(TestBase):
    @expectedFailureAll(
        oslist=["windows"], archs=["i[3-6]86", "x86_64"]
    )  # Fails to read memory from target.
    @no_debug_info_test
    def test(self):
        self.build()
        self.dbg.CreateTarget(self.getBuildArtifact("a.out"))

        self.expect_expr(
            "emptyTypePack",
            result_type="TypePack<>",
            result_children=[ValueCheck(name="a", type="int")],
        )
        self.expect_expr(
            "oneElemTypePack",
            result_type="TypePack<int>",
            result_children=[ValueCheck(name="a", type="int")],
        )
        self.expect_expr(
            "twoElemTypePack",
            result_type="TypePack<int, float>",
            result_children=[ValueCheck(name="a", type="int")],
        )

        self.expect_expr(
            "emptyAnonTypePack",
            result_type="AnonTypePack<>",
            result_children=[ValueCheck(name="b", type="int")],
        )
        self.expect_expr(
            "oneElemAnonTypePack",
            result_type="AnonTypePack<int>",
            result_children=[ValueCheck(name="b", type="int")],
        )
        self.expect_expr(
            "twoElemAnonTypePack",
            result_type="AnonTypePack<int, float>",
            result_children=[ValueCheck(name="b", type="int")],
        )

        self.expect_expr(
            "emptyAnonTypePackAfterTypeParam",
            result_type="AnonTypePackAfterTypeParam<int>",
            result_children=[ValueCheck(name="c", type="int")],
        )
        self.expect_expr(
            "oneElemAnonTypePackAfterTypeParam",
            result_type="AnonTypePackAfterTypeParam<int, float>",
            result_children=[ValueCheck(name="c", type="int")],
        )

        self.expect_expr(
            "emptyAnonTypePackAfterAnonTypeParam",
            result_type="AnonTypePackAfterAnonTypeParam<int>",
            result_children=[ValueCheck(name="d", type="float")],
        )
        self.expect_expr(
            "oneElemAnonTypePackAfterAnonTypeParam",
            result_type="AnonTypePackAfterAnonTypeParam<int, float>",
            result_children=[ValueCheck(name="d", type="float")],
        )

        self.expect_expr(
            "emptyTypePackAfterAnonTypeParam",
            result_type="TypePackAfterAnonTypeParam<int>",
            result_children=[ValueCheck(name="e", type="int")],
        )
        self.expect_expr(
            "oneElemTypePackAfterAnonTypeParam",
            result_type="TypePackAfterAnonTypeParam<int, float>",
            result_children=[ValueCheck(name="e", type="int")],
        )

        self.expect_expr(
            "emptyTypePackAfterTypeParam",
            result_type="TypePackAfterTypeParam<int>",
            result_children=[ValueCheck(name="f", type="int")],
        )
        self.expect_expr(
            "oneElemTypePackAfterTypeParam",
            result_type="TypePackAfterTypeParam<int, float>",
            result_children=[ValueCheck(name="f", type="int")],
        )

        self.expect_expr(
            "emptyAnonTypePackAfterNonTypeParam",
            result_type="AnonTypePackAfterNonTypeParam<1>",
            result_children=[ValueCheck(name="g", type="int")],
        )
        self.expect_expr(
            "oneElemAnonTypePackAfterNonTypeParam",
            result_type="AnonTypePackAfterNonTypeParam<1, int>",
            result_children=[ValueCheck(name="g", type="int")],
        )

        self.expect_expr(
            "emptyAnonTypePackAfterAnonNonTypeParam",
            result_type="AnonTypePackAfterAnonNonTypeParam<1>",
            result_children=[ValueCheck(name="h", type="float")],
        )
        self.expect_expr(
            "oneElemAnonTypePackAfterAnonNonTypeParam",
            result_type="AnonTypePackAfterAnonNonTypeParam<1, int>",
            result_children=[ValueCheck(name="h", type="float")],
        )

        self.expect_expr(
            "emptyTypePackAfterAnonNonTypeParam",
            result_type="TypePackAfterAnonNonTypeParam<1>",
            result_children=[ValueCheck(name="i", type="int")],
        )
        self.expect_expr(
            "oneElemTypePackAfterAnonNonTypeParam",
            result_type="TypePackAfterAnonNonTypeParam<1, int>",
            result_children=[ValueCheck(name="i", type="int")],
        )

        self.expect_expr(
            "emptyTypePackAfterNonTypeParam",
            result_type="TypePackAfterNonTypeParam<1>",
            result_children=[ValueCheck(name="j", type="int")],
        )
        self.expect_expr(
            "oneElemTypePackAfterNonTypeParam",
            result_type="TypePackAfterNonTypeParam<1, int>",
            result_children=[ValueCheck(name="j", type="int")],
        )



# TestSwiftPrivateDeclName.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that we correctly find private decls
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftPrivateDeclName(TestBase):
    def setUp(self):
        TestBase.setUp(self)
        self.a_source = "a.swift"
        self.a_source_spec = lldb.SBFileSpec(self.a_source)
        self.b_source = "b.swift"
        self.b_source_spec = lldb.SBFileSpec(self.b_source)

    @swiftTest
    def test_swift_private_decl_name(self):
        """Test that we correctly find private decls"""
        self.build()

        # Create the target
        target = self.dbg.CreateTarget(self.getBuildArtifact())
        self.assertTrue(target, VALID_TARGET)

        # Set the breakpoints
        a_breakpoint = target.BreakpointCreateBySourceRegex(
            'break here', self.a_source_spec)
        self.assertTrue(a_breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT)
        b_breakpoint = target.BreakpointCreateBySourceRegex(
            'break here', self.b_source_spec)
        self.assertTrue(b_breakpoint.GetNumLocations() > 0, VALID_BREAKPOINT)

        process = target.LaunchSimple(None, None, os.getcwd())
        self.assertTrue(process, PROCESS_IS_VALID)

        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, a_breakpoint)

        self.assertTrue(len(threads) == 1)

        var = self.frame().FindVariable("a")
        child_a = var.GetChildMemberWithName("a")
        child_b = var.GetChildMemberWithName("b")
        child_c = var.GetChildMemberWithName("c")
        lldbutil.check_variable(self, var, False, typename="a.S.A")
        lldbutil.check_variable(self, child_a, False, value="1")
        lldbutil.check_variable(self, child_b, False, '"hello"')
        lldbutil.check_variable(self, child_c, False, value='1.25')

        process.Continue()
        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, b_breakpoint)

        self.assertTrue(len(threads) == 1)

        var = self.frame().FindVariable("a")
        child_a = var.GetChildMemberWithName("a")
        child_b = var.GetChildMemberWithName("b")
        child_c = var.GetChildMemberWithName("c")
        lldbutil.check_variable(self, var, False, typename="a.S.A")
        lldbutil.check_variable(self, child_a, False, value="3")
        lldbutil.check_variable(self, child_b, False, '"goodbye"')
        lldbutil.check_variable(self, child_c, False, value='1.25')




"""
They may be cases where an expression will import SourceLocation and if the
SourceLocation ends up with a FileID that is a built-in we need to copy that
buffer over correctly.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestImportBuiltinFileID(TestBase):
    @skipIfDarwinEmbedded
    @add_test_categories(["gmodules", "objc"])
    def test_import_builtin_fileid(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.m", False)
        )

        self.expect(
            "expr int (*DBG_CGImageGetRenderingIntent)(void *) = ((int (*)(void *))CGImageGetRenderingIntent); DBG_CGImageGetRenderingIntent((void *)0x00000000000000);",
            substrs=["$0 = 0"],
        )



# RUN: %PYTHON %s | FileCheck %s

import gc
import io
import itertools
from mlir.ir import *
from mlir.dialects import builtin
from mlir.dialects import cf
from mlir.dialects import func


def run(f):
    print("\nTEST:", f.__name__)
    f()
    gc.collect()
    assert Context._get_live_count() == 0
    return f


# CHECK-LABEL: TEST: testBlockCreation
# CHECK: func @test(%[[ARG0:.*]]: i32 loc("arg0"), %[[ARG1:.*]]: i16 loc("arg1"))
# CHECK:   cf.br ^bb1(%[[ARG1]] : i16)
# CHECK: ^bb1(%[[PHI0:.*]]: i16 loc("middle")):
# CHECK:   cf.br ^bb2(%[[ARG0]] : i32)
# CHECK: ^bb2(%[[PHI1:.*]]: i32 loc("successor")):
# CHECK:   return
@run
def testBlockCreation():
    with Context() as ctx, Location.unknown():
        module = builtin.ModuleOp()
        with InsertionPoint(module.body):
            f_type = FunctionType.get(
                [IntegerType.get_signless(32), IntegerType.get_signless(16)], []
            )
            f_op = func.FuncOp("test", f_type)
            entry_block = f_op.add_entry_block(
                [Location.name("arg0"), Location.name("arg1")]
            )
            i32_arg, i16_arg = entry_block.arguments
            successor_block = entry_block.create_after(
                i32_arg.type, arg_locs=[Location.name("successor")]
            )
            with InsertionPoint(successor_block) as successor_ip:
                assert successor_ip.block == successor_block
                func.ReturnOp([])
            middle_block = successor_block.create_before(
                i16_arg.type, arg_locs=[Location.name("middle")]
            )

            with InsertionPoint(entry_block) as entry_ip:
                assert entry_ip.block == entry_block
                cf.BranchOp([i16_arg], dest=middle_block)

            with InsertionPoint(middle_block) as middle_ip:
                assert middle_ip.block == middle_block
                cf.BranchOp([i32_arg], dest=successor_block)
        module.print(enable_debug_info=True)
        # Ensure region back references are coherent.
        assert entry_block.region == middle_block.region == successor_block.region


# CHECK-LABEL: TEST: testBlockCreationArgLocs
@run
def testBlockCreationArgLocs():
    with Context() as ctx:
        ctx.allow_unregistered_dialects = True
        f32 = F32Type.get()
        op = Operation.create("test", regions=1, loc=Location.unknown())
        blocks = op.regions[0].blocks

        with Location.name("default_loc"):
            blocks.append(f32)
        blocks.append()
        # CHECK:      ^bb0(%{{.+}}: f32 loc("default_loc")):
        # CHECK-NEXT: ^bb1:
        op.print(enable_debug_info=True)

        try:
            blocks.append(f32)
        except RuntimeError as err:
            # CHECK: Missing loc: An MLIR function requires a Location but none was provided
            print("Missing loc:", err)

        try:
            blocks.append(f32, f32, arg_locs=[Location.unknown()])
        except ValueError as err:
            # CHECK: Wrong loc count: Expected 2 locations, got: 1
            print("Wrong loc count:", err)


# CHECK-LABEL: TEST: testFirstBlockCreation
# CHECK: func @test(%{{.*}}: f32 loc("arg_loc"))
# CHECK:   return
@run
def testFirstBlockCreation():
    with Context() as ctx, Location.unknown():
        module = builtin.ModuleOp()
        f32 = F32Type.get()
        with InsertionPoint(module.body):
            f = func.FuncOp("test", ([f32], []))
            entry_block = Block.create_at_start(
                f.operation.regions[0], [f32], [Location.name("arg_loc")]
            )
            with InsertionPoint(entry_block):
                func.ReturnOp([])

        module.print(enable_debug_info=True)
        assert module.verify()
        assert f.body.blocks[0] == entry_block


# CHECK-LABEL: TEST: testBlockMove
# CHECK:  %0 = "realop"() ({
# CHECK:  ^bb0([[ARG0:%.+]]: f32):
# CHECK:    "ret"([[ARG0]]) : (f32) -> ()
# CHECK:  }) : () -> f32
@run
def testBlockMove():
    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):
            dummy = Operation.create("dummy", regions=1)
            block = Block.create_at_start(dummy.operation.regions[0], [f32])
            with InsertionPoint(block):
                ret_op = Operation.create("ret", operands=[block.arguments[0]])
            realop = Operation.create(
                "realop", results=[r.type for r in ret_op.operands], regions=1
            )
            block.append_to(realop.operation.regions[0])
            dummy.operation.erase()
        print(module)


# CHECK-LABEL: TEST: testBlockHash
@run
def testBlockHash():
    with Context() as ctx, Location.unknown():
        ctx.allow_unregistered_dialects = True
        module = Module.create()
        f32 = F32Type.get()
        with InsertionPoint(module.body):
            dummy = Operation.create("dummy", regions=1)
            block1 = Block.create_at_start(dummy.operation.regions[0], [f32])
            block2 = Block.create_at_start(dummy.operation.regions[0], [f32])
            assert hash(block1) != hash(block2)



"""
Test that we are able to properly report a usable dynamic type
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCDynamicSBTypeTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # We'll use the test method name as the exe_name.
        self.exe_name = self.testMethodName
        # Find the line number to break inside main().
        self.main_source = "main.m"
        self.line = line_number(self.main_source, "// Set breakpoint here.")

    @skipIf(archs="i[3-6]86")
    def test_dyn(self):
        """Test that we are able to properly report a usable dynamic type."""
        d = {"EXE": self.exe_name}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)

        exe = self.getBuildArtifact(self.exe_name)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, self.main_source, self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        v_object = (
            self.frame()
            .FindVariable("object")
            .GetDynamicValue(lldb.eDynamicCanRunTarget)
        )
        v_base = (
            self.frame().FindVariable("base").GetDynamicValue(lldb.eDynamicCanRunTarget)
        )
        self.assertEqual(
            v_object.GetTypeName(),
            "MyDerivedClass *",
            "The NSObject is properly type-named",
        )
        self.assertEqual(
            v_base.GetTypeName(), "MyDerivedClass *", "The Base is properly type-named"
        )
        object_type = v_object.GetType()
        base_type = v_base.GetType()
        self.assertEqual(
            object_type.GetName(),
            "MyDerivedClass *",
            "The dynamic SBType for NSObject is for the correct type",
        )
        self.assertEqual(
            base_type.GetName(),
            "MyDerivedClass *",
            "The dynamic SBType for Base is for the correct type",
        )
        object_pointee_type = object_type.GetPointeeType()
        base_pointee_type = base_type.GetPointeeType()
        self.assertEqual(
            object_pointee_type.GetName(),
            "MyDerivedClass",
            "The dynamic type for NSObject figures out its pointee type just fine",
        )
        self.assertEqual(
            base_pointee_type.GetName(),
            "MyDerivedClass",
            "The dynamic type for Base figures out its pointee type just fine",
        )

        self.assertEqual(
            object_pointee_type.GetDirectBaseClassAtIndex(0).GetName(),
            "MyBaseClass",
            "The dynamic type for NSObject can go back to its base class",
        )
        self.assertEqual(
            base_pointee_type.GetDirectBaseClassAtIndex(0).GetName(),
            "MyBaseClass",
            "The dynamic type for Base can go back to its base class",
        )

        self.assertEqual(
            object_pointee_type.GetDirectBaseClassAtIndex(0)
            .GetType()
            .GetDirectBaseClassAtIndex(0)
            .GetName(),
            "NSObject",
            "The dynamic type for NSObject can go up the hierarchy",
        )
        self.assertEqual(
            base_pointee_type.GetDirectBaseClassAtIndex(0)
            .GetType()
            .GetDirectBaseClassAtIndex(0)
            .GetName(),
            "NSObject",
            "The dynamic type for Base can go up the hierarchy",
        )

        self.assertEqual(
            object_pointee_type.GetNumberOfFields(),
            2,
            "The dynamic type for NSObject has 2 fields",
        )
        self.assertEqual(
            base_pointee_type.GetNumberOfFields(),
            2,
            "The dynamic type for Base has 2 fields",
        )




"""
Test that evaluating expressions works on backward interop mode.
"""
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


class TestSwiftBackwardInteropExpressions(TestBase):

    @skipIfLinux
    @swiftTest
    def test_func_step_in(self):
        self.build()
        self.runCmd('setting set target.experimental.swift-enable-cxx-interop true')
        _, _, _, _ = lldbutil.run_to_source_breakpoint(
            self, 'Break here', lldb.SBFileSpec('main.cpp'))
        self.expect('expr swiftFunc()', substrs=["Inside a Swift function"])
        self.expect('expr swiftClass.swiftMethod()', substrs=["Inside a Swift method"])
        self.expect('expr a::SwiftClass::swiftStaticMethod()', substrs=["In a Swift static method"])
        self.expect('expr swiftClass.getSwiftProperty()', substrs=["This is a class with properties"])
        self.expect('expr swiftSubclassAsClass.overrideableMethod()', substrs=["In subclass"])



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

# This file contains the utilities to process sparse tensor outputs.

from typing import Callable, Dict, Sequence, Tuple
import ctypes
import functools
import numpy as np
import os

# Import MLIR related modules.
from mlir import execution_engine
from mlir import ir
from mlir import runtime
from mlir.dialects import sparse_tensor

from . import mlir_sparse_compiler

# Type aliases for type annotation.
_SupportFunc = Callable[..., None]
_SupportFuncLocator = Callable[[np.dtype], Tuple[_SupportFunc, _SupportFunc]]

# The name for the environment variable that provides the full path for the
# supporting library.
_SUPPORTLIB_ENV_VAR = "SUPPORTLIB"
# The default supporting library if the environment variable is not provided.
_DEFAULT_SUPPORTLIB = "libmlir_c_runner_utils.so"

# The JIT compiler optimization level.
_OPT_LEVEL = 2
# The entry point to the JIT compiled program.
_ENTRY_NAME = "main"


@functools.lru_cache()
def _get_support_lib_name() -> str:
    """Gets the string name for the supporting C shared library."""
    return os.getenv(_SUPPORTLIB_ENV_VAR, _DEFAULT_SUPPORTLIB)


@functools.lru_cache()
def _get_sparse_compiler() -> mlir_sparse_compiler.SparseCompiler:
    """Gets the MLIR sparse compiler with default setting."""
    return mlir_sparse_compiler.SparseCompiler(
        options="", opt_level=_OPT_LEVEL, shared_libs=[_get_support_lib_name()]
    )


def _record_support_funcs(
    ty: np.dtype,
    to_func: _SupportFunc,
    from_func: _SupportFunc,
    ty_to_funcs: Dict[np.dtype, Tuple[_SupportFunc, _SupportFunc]],
) -> None:
    """Records the two supporting functions for a given data type."""
    to_func.restype = ctypes.c_void_p
    from_func.restype = ctypes.c_void_p
    ty_to_funcs[ty] = (to_func, from_func)


@functools.lru_cache()
def _get_support_func_locator() -> _SupportFuncLocator:
    """Constructs a function to locate the supporting functions for a data type.

    Loads the supporting C shared library with the needed routines. Constructs a
    dictionary from the supported data types to the routines for the data types,
    and then a function to look up the dictionary for a given data type.

    The name of the supporting C shared library is either provided by an
    an environment variable or a default value.

    Returns:
      The function to look up the supporting functions for a given data type.

    Raises:
      OSError: If there is any problem in loading the shared library.
      ValueError: If the shared library doesn't contain the needed routines.
    """
    # This raises OSError exception if there is any problem in loading the shared
    # library.
    c_lib = ctypes.CDLL(_get_support_lib_name())

    type_to_funcs = {}
    try:
        support_types = [
            (
                np.int8,
                c_lib.convertToMLIRSparseTensorI8,
                c_lib.convertFromMLIRSparseTensorI8,
            ),
            (
                np.int16,
                c_lib.convertToMLIRSparseTensorI16,
                c_lib.convertFromMLIRSparseTensorI16,
            ),
            (
                np.int32,
                c_lib.convertToMLIRSparseTensorI32,
                c_lib.convertFromMLIRSparseTensorI32,
            ),
            (
                np.int64,
                c_lib.convertToMLIRSparseTensorI64,
                c_lib.convertFromMLIRSparseTensorI64,
            ),
            (
                np.float16,
                c_lib.convertToMLIRSparseTensorF16,
                c_lib.convertFromMLIRSparseTensorF16,
            ),
            (
                np.float32,
                c_lib.convertToMLIRSparseTensorF32,
                c_lib.convertFromMLIRSparseTensorF32,
            ),
            (
                np.float64,
                c_lib.convertToMLIRSparseTensorF64,
                c_lib.convertFromMLIRSparseTensorF64,
            ),
            (
                np.complex64,
                c_lib.convertToMLIRSparseTensorC32,
                c_lib.convertFromMLIRSparseTensorC32,
            ),
            (
                np.complex128,
                c_lib.convertToMLIRSparseTensorC64,
                c_lib.convertFromMLIRSparseTensorC64,
            ),
        ]
    except Exception as e:
        raise ValueError(f"Missing supporting function: {e}") from e
    for i, info in enumerate(support_types):
        _record_support_funcs(info[0], info[1], info[2], type_to_funcs)

    def get_support_funcs(ty: np.dtype):
        funcs = type_to_funcs[ty]
        assert funcs is not None
        return funcs

    return get_support_funcs


def sparse_tensor_to_coo_tensor(
    sparse_tensor: ctypes.c_void_p,
    dtype: np.dtype,
) -> Tuple[int, int, np.ndarray, np.ndarray, np.ndarray]:
    """Converts an MLIR sparse tensor to a COO-flavored format tensor.

    Args:
       sparse_tensor: A ctypes.c_void_p to the MLIR sparse tensor descriptor.
       dtype: The numpy data type for the tensor elements.

    Returns:
      A tuple that contains the following values for the COO-flavored format
      tensor:
      rank: An integer for the rank of the tensor.
      nse: An integer for the number of non-zero values in the tensor.
      shape: A 1D numpy array of integers, for the shape of the tensor.
      values: A 1D numpy array, for the non-zero values in the tensor.
      indices: A 2D numpy array of integers, representing the indices for the
        non-zero values in the tensor.

    Raises:
      OSError: If there is any problem in loading the shared library.
      ValueError: If the shared library doesn't contain the needed routines.
    """
    convert_from = _get_support_func_locator()(dtype)[1]
    rank = ctypes.c_ulonglong(0)
    nse = ctypes.c_ulonglong(0)
    shape = ctypes.POINTER(ctypes.c_ulonglong)()

    values = ctypes.POINTER(runtime.as_ctype(np.dtype(dtype)))()
    indices = ctypes.POINTER(ctypes.c_ulonglong)()
    convert_from(
        sparse_tensor,
        ctypes.byref(rank),
        ctypes.byref(nse),
        ctypes.byref(shape),
        ctypes.byref(values),
        ctypes.byref(indices),
    )

    # Convert the returned values to the corresponding numpy types.
    shape = np.ctypeslib.as_array(shape, shape=[rank.value])
    values = runtime.to_numpy(np.ctypeslib.as_array(values, shape=[nse.value]))
    indices = np.ctypeslib.as_array(indices, shape=[nse.value, rank.value])
    return rank.value, nse.value, shape, values, indices


def coo_tensor_to_sparse_tensor(
    np_shape: np.ndarray,
    np_values: np.ndarray,
    np_indices: np.ndarray,
    np_perm: np.ndarray,
    np_sparse: np.ndarray,
) -> int:
    """Converts a COO-flavored format sparse tensor to an MLIR sparse tensor.

    Args:
       np_shape: A 1D numpy array of integers, for the shape of the tensor.
       np_values: A 1D numpy array, for the non-zero values in the tensor.
       np_indices: A 2D numpy array of integers, representing the indices for the
         non-zero values in the tensor.
       np_perm: A 1D numpy array of integers, representing the storage ordering
         for the dimensions.
       np_sparse: A 1D numpy array of uint8, representing the sparsity values
         for the dimensions.

    Returns:
       An integer for the non-null ctypes.c_void_p to the MLIR sparse tensor
       descriptor.

    Raises:
      OSError: If there is any problem in loading the shared library.
      ValueError: If the shared library doesn't contain the needed routines.
    """

    r = len(np_shape)
    rank = ctypes.c_ulonglong(r)
    nse = ctypes.c_ulonglong(len(np_values))
    shape = np_shape.ctypes.data_as(ctypes.POINTER(ctypes.c_ulonglong))
    values = np_values.ctypes.data_as(
        ctypes.POINTER(runtime.as_ctype(np.dtype(np_values.dtype)))
    )
    indices = np_indices.ctypes.data_as(ctypes.POINTER(ctypes.c_ulonglong))

    perm = np_perm.ctypes.data_as(ctypes.POINTER(ctypes.c_ulonglong))
    sparse = np_sparse.ctypes.data_as(ctypes.POINTER(ctypes.c_uint8))

    convert_to = _get_support_func_locator()(np_values.dtype.type)[0]
    ptr = convert_to(rank, nse, shape, values, indices, perm, sparse)
    assert ptr is not None, "Problem with calling convertToMLIRSparseTensorF64"
    return ptr


def compile_and_build_engine(module: ir.Module) -> execution_engine.ExecutionEngine:
    """Compiles an MLIR module and builds a JIT execution engine.

    Args:
      module: The MLIR module.

    Returns:
      A JIT execution engine for the MLIR module.

    """
    return _get_sparse_compiler().compile_and_jit(module)


class _SparseTensorDescriptor(ctypes.Structure):
    """A C structure for an MLIR sparse tensor."""

    _fields_ = [
        # A pointer for the MLIR sparse tensor storage.
        ("storage", ctypes.POINTER(ctypes.c_ulonglong)),
        # An MLIR MemRef descriptor for the shape of the sparse tensor.
        ("shape", runtime.make_nd_memref_descriptor(1, ctypes.c_ulonglong)),
    ]


def _output_one_dim(dim: int, rank: int, shape: str, type: str) -> str:
    """Produces the MLIR text code to output the size for the given dimension."""
    return f"""
  %c{dim} = arith.constant {dim} : index
  %d{dim} = tensor.dim %t, %c{dim} : tensor<{shape}x{type}, #enc>
  memref.store %d{dim}, %b[%c{dim}] : memref<{rank}xindex>
"""


# TODO: With better support from MLIR, we may improve the current implementation
# by doing the following:
# (1) Use Python code to generate the kernel instead of doing MLIR text code
#     stitching.
# (2) Use scf.for instead of an unrolled loop to write out the dimension sizes
#     when tensor.dim supports non-constant dimension value.
def _get_create_sparse_tensor_kernel(
    sparsity_codes: Sequence[sparse_tensor.DimLevelType], type: str
) -> str:
    """Creates an MLIR text kernel to contruct a sparse tensor from a file.

    The kernel returns a _SparseTensorDescriptor structure.
    """
    rank = len(sparsity_codes)

    # Use ? to represent a dimension in the dynamic shape string representation.
    shape = "x".join(map(lambda d: "?", range(rank)))

    # Convert the encoded sparsity values to a string representation.
    sparsity = ", ".join(
        map(lambda s: '"compressed"' if s.value else '"dense"', sparsity_codes)
    )

    # Get the MLIR text code to write the dimension sizes to the output buffer.
    output_dims = "\n".join(
        map(lambda d: _output_one_dim(d, rank, shape, type), range(rank))
    )

    # Return the MLIR text kernel.
    return f"""
!Ptr = !llvm.ptr<i8>
#enc = #sparse_tensor.encoding<{{
  lvlTypes = [ {sparsity} ]
}}>
func.func @{_ENTRY_NAME}(%filename: !Ptr) -> (tensor<{shape}x{type}, #enc>, memref<{rank}xindex>)
attributes {{ llvm.emit_c_interface }} {{
  %t = sparse_tensor.new %filename : !Ptr to tensor<{shape}x{type}, #enc>
  %b = memref.alloc() : memref<{rank}xindex>
  {output_dims}
  return %t, %b : tensor<{shape}x{type}, #enc>, memref<{rank}xindex>
}}"""


def create_sparse_tensor(
    filename: str, sparsity: Sequence[sparse_tensor.DimLevelType], type: str
) -> Tuple[ctypes.c_void_p, np.ndarray]:
    """Creates an MLIR sparse tensor from the input file.

    Args:
      filename: A string for the name of the file that contains the tensor data in
        a COO-flavored format.
      sparsity: A sequence of DimLevelType values, one for each dimension of the
        tensor.

    Returns:
      A Tuple containing the following values:
      storage: A ctypes.c_void_p for the MLIR sparse tensor storage.
      shape: A 1D numpy array of integers, for the shape of the tensor.

    Raises:
      OSError: If there is any problem in loading the supporting C shared library.
      ValueError:  If the shared library doesn't contain the needed routine.
    """
    with ir.Context() as ctx, ir.Location.unknown():
        module = _get_create_sparse_tensor_kernel(sparsity, type)
        module = ir.Module.parse(module)
        engine = compile_and_build_engine(module)

    # A sparse tensor descriptor to receive the kernel result.
    c_tensor_desc = _SparseTensorDescriptor()
    # Convert the filename to a byte stream.
    c_filename = ctypes.c_char_p(bytes(filename, "utf-8"))

    arg_pointers = [
        ctypes.byref(ctypes.pointer(c_tensor_desc)),
        ctypes.byref(c_filename),
    ]

    # Invoke the execution engine to run the module and return the result.
    engine.invoke(_ENTRY_NAME, *arg_pointers)
    shape = runtime.ranked_memref_to_numpy(ctypes.pointer(c_tensor_desc.shape))
    return c_tensor_desc.storage, shape


# TODO: With better support from MLIR, we may improve the current implementation
# by using Python code to generate the kernel instead of doing MLIR text code
# stitching.
def _get_output_sparse_tensor_kernel(
    sparsity_codes: Sequence[sparse_tensor.DimLevelType], type: str
) -> str:
    """Creates an MLIR text kernel to output a sparse tensor to a file.

    The kernel returns void.
    """
    rank = len(sparsity_codes)

    # Use ? to represent a dimension in the dynamic shape string representation.
    shape = "x".join(map(lambda d: "?", range(rank)))

    # Convert the encoded sparsity values to a string representation.
    sparsity = ", ".join(
        map(lambda s: '"compressed"' if s.value else '"dense"', sparsity_codes)
    )

    # Return the MLIR text kernel.
    return f"""
!Ptr = !llvm.ptr<i8>
#enc = #sparse_tensor.encoding<{{
  lvlTypes = [ {sparsity} ]
}}>
func.func @{_ENTRY_NAME}(%t: tensor<{shape}x{type}, #enc>, %filename: !Ptr)
attributes {{ llvm.emit_c_interface }} {{
  sparse_tensor.out %t, %filename : tensor<{shape}x{type}, #enc>, !Ptr
  func.return
}}"""


def output_sparse_tensor(
    tensor: ctypes.c_void_p,
    filename: str,
    sparsity: Sequence[sparse_tensor.DimLevelType],
    type: str,
) -> None:
    """Outputs an MLIR sparse tensor to the given file.

    Args:
      tensor: A C pointer to the MLIR sparse tensor.
      filename: A string for the name of the file that contains the tensor data in
        a COO-flavored format.
      sparsity: A sequence of DimLevelType values, one for each dimension of the
        tensor.
      type: The MLIR string for the data type.

    Raises:
      OSError: If there is any problem in loading the supporting C shared library.
      ValueError:  If the shared library doesn't contain the needed routine.
    """
    with ir.Context() as ctx, ir.Location.unknown():
        module = _get_output_sparse_tensor_kernel(sparsity, type)
        module = ir.Module.parse(module)
        engine = compile_and_build_engine(module)

    # Convert the filename to a byte stream.
    c_filename = ctypes.c_char_p(bytes(filename, "utf-8"))

    arg_pointers = [
        ctypes.byref(ctypes.cast(tensor, ctypes.c_void_p)),
        ctypes.byref(c_filename),
    ]

    # Invoke the execution engine to run the module and return the result.
    engine.invoke(_ENTRY_NAME, *arg_pointers)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import unittest2


class TestSwiftEmbeddedNestedFrameVariable(TestBase):
    @skipUnlessDarwin
    @swiftTest
    def test(self):
        self.build()
        self.implementation()

    @skipUnlessDarwin
    @swiftTest
    def test_without_ast(self):
        """Run the test turning off instantion of  Swift AST contexts in order to ensure that all type information comes from DWARF"""
        self.build()
        self.runCmd("setting set symbols.swift-enable-ast-context false")
        self.implementation()

    def implementation(self):
        self.runCmd("setting set symbols.swift-enable-full-dwarf-debugging true")

        target, process, thread, _ = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.swift")
        )
        frame = thread.frames[0]
        self.assertTrue(frame, "Frame 0 is valid.")

        s4 = frame.FindVariable("s4")
        t = s4.GetChildMemberWithName("t")
        lldbutil.check_variable(self, t, False, value='839')




"""
Test setting breakpoints using a scripted resolver
"""

import os
import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestScriptedResolver(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24528")
    def test_scripted_resolver(self):
        """Use a scripted resolver to set a by symbol name breakpoint"""
        self.build()
        self.do_test()

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24528")
    def test_search_depths(self):
        """Make sure we are called at the right depths depending on what we return
        from __get_depth__"""
        self.build()
        self.do_test_depths()

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24528")
    def test_command_line(self):
        """Test setting a resolver breakpoint from the command line"""
        self.build()
        self.do_test_cli()

    def test_bad_command_lines(self):
        """Make sure we get appropriate errors when we give invalid key/value
        options"""
        self.build()
        self.do_test_bad_options()

    def test_copy_from_dummy_target(self):
        """Make sure we don't crash during scripted breakpoint copy from dummy target"""
        self.build()
        self.do_test_copy_from_dummy_target()

    def make_target_and_import(self):
        target = self.make_target()
        self.import_resolver_script()
        return target

    def make_target(self):
        return lldbutil.run_to_breakpoint_make_target(self)

    def import_resolver_script(self):
        interp = self.dbg.GetCommandInterpreter()
        error = lldb.SBError()

        script_name = os.path.join(self.getSourceDir(), "resolver.py")
        source_name = os.path.join(self.getSourceDir(), "main.c")

        command = "command script import " + script_name
        result = lldb.SBCommandReturnObject()
        interp.HandleCommand(command, result)
        self.assertTrue(
            result.Succeeded(), "com scr imp failed: %s" % (result.GetError())
        )

    def make_extra_args(self):
        json_string = '{"symbol":"break_on_me", "test1": "value1"}'
        json_stream = lldb.SBStream()
        json_stream.Print(json_string)
        extra_args = lldb.SBStructuredData()
        error = extra_args.SetFromJSON(json_stream)
        self.assertSuccess(error, "Error making SBStructuredData")
        return extra_args

    def do_test(self):
        """This reads in a python file and sets a breakpoint using it."""

        target = self.make_target_and_import()
        extra_args = self.make_extra_args()

        file_list = lldb.SBFileSpecList()
        module_list = lldb.SBFileSpecList()

        # Make breakpoints with this resolver using different filters, first ones that will take:
        right = []
        # one with no file or module spec - this one should fire:
        right.append(
            target.BreakpointCreateFromScript(
                "resolver.Resolver", extra_args, module_list, file_list
            )
        )

        # one with the right source file and no module - should also fire:
        file_list.Append(lldb.SBFileSpec("main.c"))
        right.append(
            target.BreakpointCreateFromScript(
                "resolver.Resolver", extra_args, module_list, file_list
            )
        )
        # Make sure the help text shows up in the "break list" output:
        self.expect(
            "break list",
            substrs=["I am a python breakpoint resolver"],
            msg="Help is listed in break list",
        )

        # one with the right source file and right module - should also fire:
        module_list.Append(lldb.SBFileSpec("a.out"))
        right.append(
            target.BreakpointCreateFromScript(
                "resolver.Resolver", extra_args, module_list, file_list
            )
        )

        # And one with no source file but the right module:
        file_list.Clear()
        right.append(
            target.BreakpointCreateFromScript(
                "resolver.Resolver", extra_args, module_list, file_list
            )
        )

        # Make sure these all got locations:
        for i in range(0, len(right)):
            self.assertTrue(
                right[i].GetNumLocations() >= 1, "Breakpoint %d has no locations." % (i)
            )

        # Now some ones that won't take:

        module_list.Clear()
        file_list.Clear()
        wrong = []

        # one with the wrong module - should not fire:
        module_list.Append(lldb.SBFileSpec("noSuchModule"))
        wrong.append(
            target.BreakpointCreateFromScript(
                "resolver.Resolver", extra_args, module_list, file_list
            )
        )

        # one with the wrong file - also should not fire:
        file_list.Clear()
        module_list.Clear()
        file_list.Append(lldb.SBFileSpec("noFileOfThisName.xxx"))
        wrong.append(
            target.BreakpointCreateFromScript(
                "resolver.Resolver", extra_args, module_list, file_list
            )
        )

        # Now make sure the CU level iteration obeys the file filters:
        file_list.Clear()
        module_list.Clear()
        file_list.Append(lldb.SBFileSpec("no_such_file.xxx"))
        wrong.append(
            target.BreakpointCreateFromScript(
                "resolver.ResolverCUDepth", extra_args, module_list, file_list
            )
        )

        # And the Module filters:
        file_list.Clear()
        module_list.Clear()
        module_list.Append(lldb.SBFileSpec("NoSuchModule.dylib"))
        wrong.append(
            target.BreakpointCreateFromScript(
                "resolver.ResolverCUDepth", extra_args, module_list, file_list
            )
        )

        # Now make sure the Function level iteration obeys the file filters:
        file_list.Clear()
        module_list.Clear()
        file_list.Append(lldb.SBFileSpec("no_such_file.xxx"))
        wrong.append(
            target.BreakpointCreateFromScript(
                "resolver.ResolverFuncDepth", extra_args, module_list, file_list
            )
        )

        # And the Module filters:
        file_list.Clear()
        module_list.Clear()
        module_list.Append(lldb.SBFileSpec("NoSuchModule.dylib"))
        wrong.append(
            target.BreakpointCreateFromScript(
                "resolver.ResolverFuncDepth", extra_args, module_list, file_list
            )
        )

        # Make sure these didn't get locations:
        for i in range(0, len(wrong)):
            self.assertEqual(
                wrong[i].GetNumLocations(), 0, "Breakpoint %d has locations." % (i)
            )

        # Now run to main and ensure we hit the breakpoints we should have:

        lldbutil.run_to_breakpoint_do_run(self, target, right[0])

        # Test the hit counts:
        for i in range(0, len(right)):
            self.assertEqual(
                right[i].GetHitCount(), 1, "Breakpoint %d has the wrong hit count" % (i)
            )

        for i in range(0, len(wrong)):
            self.assertEqual(
                wrong[i].GetHitCount(), 0, "Breakpoint %d has the wrong hit count" % (i)
            )

    def do_test_depths(self):
        """This test uses a class variable in resolver.Resolver which gets set to 1 if we saw
        compile unit and 2 if we only saw modules.  If the search depth is module, you get passed just
        the modules with no comp_unit.  If the depth is comp_unit you get comp_units.  So we can use
        this to test that our callback gets called at the right depth."""

        target = self.make_target_and_import()
        extra_args = self.make_extra_args()

        file_list = lldb.SBFileSpecList()
        module_list = lldb.SBFileSpecList()
        module_list.Append(lldb.SBFileSpec("a.out"))

        # Make a breakpoint that has no __get_depth__, check that that is converted to eSearchDepthModule:
        bkpt = target.BreakpointCreateFromScript(
            "resolver.Resolver", extra_args, module_list, file_list
        )
        self.assertTrue(bkpt.GetNumLocations() > 0, "Resolver got no locations.")
        self.expect(
            "script print(resolver.Resolver.got_files)",
            substrs=["2"],
            msg="Was only passed modules",
        )

        # Make a breakpoint that asks for modules, check that we didn't get any files:
        bkpt = target.BreakpointCreateFromScript(
            "resolver.ResolverModuleDepth", extra_args, module_list, file_list
        )
        self.assertTrue(
            bkpt.GetNumLocations() > 0, "ResolverModuleDepth got no locations."
        )
        self.expect(
            "script print(resolver.Resolver.got_files)",
            substrs=["2"],
            msg="Was only passed modules",
        )

        # Make a breakpoint that asks for compile units, check that we didn't get any files:
        bkpt = target.BreakpointCreateFromScript(
            "resolver.ResolverCUDepth", extra_args, module_list, file_list
        )
        self.assertTrue(bkpt.GetNumLocations() > 0, "ResolverCUDepth got no locations.")
        self.expect(
            "script print(resolver.Resolver.got_files)",
            substrs=["1"],
            msg="Was passed compile units",
        )

        # Make a breakpoint that returns a bad value - we should convert that to "modules" so check that:
        bkpt = target.BreakpointCreateFromScript(
            "resolver.ResolverBadDepth", extra_args, module_list, file_list
        )
        self.assertTrue(
            bkpt.GetNumLocations() > 0, "ResolverBadDepth got no locations."
        )
        self.expect(
            "script print(resolver.Resolver.got_files)",
            substrs=["2"],
            msg="Was only passed modules",
        )

        # Make a breakpoint that searches at function depth:
        bkpt = target.BreakpointCreateFromScript(
            "resolver.ResolverFuncDepth", extra_args, module_list, file_list
        )
        self.assertTrue(
            bkpt.GetNumLocations() > 0, "ResolverFuncDepth got no locations."
        )
        self.expect(
            "script print(resolver.Resolver.got_files)",
            substrs=["3"],
            msg="Was only passed modules",
        )
        self.expect(
            "script print(resolver.Resolver.func_list)",
            substrs=["test_func", "break_on_me", "main"],
            msg="Saw all the functions",
        )

    def do_test_cli(self):
        target = self.make_target_and_import()

        lldbutil.run_break_set_by_script(
            self, "resolver.Resolver", extra_options="-k symbol -v break_on_me"
        )

        # Make sure setting a resolver breakpoint doesn't pollute further breakpoint setting
        # by checking the description of a regular file & line breakpoint to make sure it
        # doesn't mention the Python Resolver function:
        bkpt_no = lldbutil.run_break_set_by_file_and_line(self, "main.c", 12)
        bkpt = target.FindBreakpointByID(bkpt_no)
        strm = lldb.SBStream()
        bkpt.GetDescription(strm, False)
        used_resolver = "I am a python breakpoint resolver" in strm.GetData()
        self.assertFalse(
            used_resolver,
            "Found the resolver description in the file & line breakpoint description.",
        )

        # Also make sure the breakpoint was where we expected:
        bp_loc = bkpt.GetLocationAtIndex(0)
        bp_sc = bp_loc.GetAddress().GetSymbolContext(lldb.eSymbolContextEverything)
        bp_se = bp_sc.GetLineEntry()
        self.assertEqual(bp_se.GetLine(), 12, "Got the right line number")
        self.assertEqual(
            bp_se.GetFileSpec().GetFilename(), "main.c", "Got the right filename"
        )

    def do_test_bad_options(self):
        target = self.make_target_and_import()

        self.expect(
            "break set -P resolver.Resolver -k a_key",
            error=True,
            msg="Missing value at end",
            substrs=['Key: "a_key" missing value'],
        )
        self.expect(
            "break set -P resolver.Resolver -v a_value",
            error=True,
            msg="Missing key at end",
            substrs=['Value: "a_value" missing matching key'],
        )
        self.expect(
            "break set -P resolver.Resolver -v a_value -k a_key -v another_value",
            error=True,
            msg="Missing key among args",
            substrs=['Value: "a_value" missing matching key'],
        )
        self.expect(
            "break set -P resolver.Resolver -k a_key -k a_key -v another_value",
            error=True,
            msg="Missing value among args",
            substrs=['Key: "a_key" missing value'],
        )

    def do_test_copy_from_dummy_target(self):
        # Import breakpoint scripted resolver.
        self.import_resolver_script()

        # Create a scripted breakpoint.
        self.runCmd(
            "breakpoint set -P resolver.Resolver -k symbol -v break_on_me",
            BREAKPOINT_CREATED,
        )

        # This is the function to remove breakpoints from the dummy target
        # to get a clean state for the next test case.
        def cleanup():
            self.runCmd("breakpoint delete -D -f", check=False)
            self.runCmd("breakpoint list", check=False)

        # Execute the cleanup function during test case tear down.
        self.addTearDownHook(cleanup)

        # Check that target creating doesn't crash.
        target = self.make_target()



"""
Test that loading of dependents works correctly for all the potential
combinations.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


@skipIfWindows  # Windows deals differently with shared libs.
class TargetDependentsTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        TestBase.setUp(self)
        self.build()

    def has_exactly_one_image(self, matching, msg=""):
        self.expect(
            "image list",
            "image list should contain at least one image",
            substrs=["[  0]"],
        )
        should_match = not matching
        self.expect("image list", msg, matching=should_match, substrs=["[  1]"])

    @expectedFailureAll(
        oslist=["freebsd", "linux", "netbsd"],
        bugnumber="llvm.org/pr48372",
        triple=no_match(".*-android"),
    )
    def test_dependents_implicit_default_exe(self):
        """Test default behavior"""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("target create  " + exe, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(False)

    @expectedFailureAll(
        oslist=["freebsd", "linux", "netbsd"],
        bugnumber="llvm.org/pr48372",
        triple=no_match(".*-android"),
    )
    def test_dependents_explicit_default_exe(self):
        """Test default behavior"""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("target create -ddefault " + exe, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(False)

    def test_dependents_explicit_true_exe(self):
        """Test default behavior"""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("target create -dtrue " + exe, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(True)

    @expectedFailureAll(
        oslist=["freebsd", "linux", "netbsd"],
        bugnumber="llvm.org/pr48372",
        triple=no_match(".*-android"),
    )
    def test_dependents_explicit_false_exe(self):
        """Test default behavior"""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("target create -dfalse " + exe, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(False)

    def test_dependents_implicit_false_exe(self):
        """Test default behavior"""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("target create  -d " + exe, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(True)

    @expectedFailureAndroid  # android will return mutiple images
    def test_dependents_implicit_default_lib(self):
        ctx = self.platformContext
        dylibName = ctx.shlib_prefix + "load_a." + ctx.shlib_extension
        lib = self.getBuildArtifact(dylibName)
        self.runCmd("target create " + lib, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(True)

    def test_dependents_explicit_default_lib(self):
        ctx = self.platformContext
        dylibName = ctx.shlib_prefix + "load_a." + ctx.shlib_extension
        lib = self.getBuildArtifact(dylibName)
        self.runCmd("target create -ddefault " + lib, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(True)

    def test_dependents_explicit_true_lib(self):
        ctx = self.platformContext
        dylibName = ctx.shlib_prefix + "load_a." + ctx.shlib_extension
        lib = self.getBuildArtifact(dylibName)
        self.runCmd("target create -dtrue " + lib, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(True)

    @expectedFailureAll(
        oslist=["freebsd", "linux", "netbsd"],
        bugnumber="llvm.org/pr48372",
        triple=no_match(".*-android"),
    )
    def test_dependents_explicit_false_lib(self):
        ctx = self.platformContext
        dylibName = ctx.shlib_prefix + "load_a." + ctx.shlib_extension
        lib = self.getBuildArtifact(dylibName)
        self.runCmd("target create -dfalse " + lib, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(False)

    def test_dependents_implicit_false_lib(self):
        ctx = self.platformContext
        dylibName = ctx.shlib_prefix + "load_a." + ctx.shlib_extension
        lib = self.getBuildArtifact(dylibName)
        self.runCmd("target create -d " + lib, CURRENT_EXECUTABLE_SET)
        self.has_exactly_one_image(True)



#!/usr/bin/env python
"""
Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
See https://llvm.org/LICENSE.txt for license information.
SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

Prepares language bindings for LLDB build process.  Run with --help
to see a description of the supported command line arguments.
"""

# Python modules:
import argparse
import logging
import os
import platform
import sys


def prepare_binding_for_language(bindings_dir, script_lang, options):
    """Prepares the binding for a specific language.

    @param bindings_dir the full path to the bindings source directory.
    @param script_lang the name of the script language.  Should be a child
    directory within the bindings dir, and should contain a
    prepare_bindings_{script_lang}.py script file in it.
    @param options the dictionary of parsed command line options.

    There is no return value.  If it returns, the process succeeded; otherwise,
    the process will exit where it fails.
    """
    # Ensure the language-specific prepare module exists.
    script_name = "prepare_binding_{}.py".format(script_lang)
    lang_path = os.path.join(bindings_dir, script_lang)
    script_path = os.path.join(lang_path, script_name)
    if not os.path.exists(script_path):
        logging.error(
            "failed to find prepare script for language '%s' at '%s'",
            script_lang,
            script_path)
        sys.exit(-9)

    # Include this language-specific directory in the Python search
    # path.
    sys.path.append(os.path.normcase(lang_path))

    # Execute the specific language script
    module_name = os.path.splitext(script_name)[0]
    module = __import__(module_name)
    module.main(options)

    # Remove the language-specific directory from the Python search path.
    sys.path.remove(os.path.normcase(lang_path))


def prepare_all_bindings(options):
    """Prepares bindings for each of the languages supported.

    @param options the parsed arguments from the command line

    @return the exit value for the program. 0 is success, all othes
    indicate some kind of failure.
    """
    # Check for the existence of the SWIG bindings folder
    bindings_dir = os.path.join(options.src_root, "bindings")
    if not os.path.exists(bindings_dir):
        logging.error("failed to find bindings dir: '%s'", bindings_dir)
        sys.exit(-8)

    child_dirs = ["python"]

    # Iterate script directory find any script language directories
    for script_lang in child_dirs:
        logging.info("executing language script for: '%s'", script_lang)
        prepare_binding_for_language(bindings_dir, script_lang, options)


def process_args(args):
    """Returns options processed from the provided command line.

    @param args the command line to process.
    """

    # Setup the parser arguments that are accepted.
    parser = argparse.ArgumentParser(
        description="Prepare language bindings for LLDB build.")

    # Arguments to control logging verbosity.
    parser.add_argument(
        "--debug", "-d",
        action="store_true",
        help="Set program logging level to DEBUG.")
    parser.add_argument(
        "--verbose", "-v",
        action="count",
        default=0,
        help=(
            "Increase logging verbosity level.  Default: only error and "
            "higher are displayed.  Each -v increases level of verbosity."))

    # Arguments to control whether we're building an OS X-style
    # framework.  This is the opposite of the older "-m" (makefile)
    # option.
    parser.add_argument(
        "--config-build-dir",
        "--cfgBldDir",
        help=(
            "Configuration build dir, will use python module path "
            "if unspecified."))
    parser.add_argument(
        "--find-swig",
        action="store_true",
        help=(
            "Indicates the swig executable should be searched for "
            "if not eplicitly provided.  Either this or the explicit "
            "swig executable option must be provided."))
    parser.add_argument(
        "--framework",
        action="store_true",
        help="Prepare as OS X-style framework.")
    parser.add_argument(
        "--generate-dependency-file",
        "-M",
        action="store_true",
        help="Make the dependency (.d) file for the wrappers.")
    parser.add_argument(
        "--prefix",
        help="Override path where the LLDB module is placed.")
    parser.add_argument(
        "--src-root",
        "--srcRoot",
        "-s",
        # Default to the parent directory of this script's directory.
        default=os.path.abspath(
            os.path.join(
                os.path.dirname(os.path.realpath(__file__)),
                os.path.pardir)),
        help="Specifies the LLDB source root directory.")
    parser.add_argument(
        "--build-root",
        "--buildRoot",
        "-b",
        help="Specifies the LLDB build root directory.")
    parser.add_argument(
        "--swig-executable",
        "--swigExecutable",
        help="Path to the swig executable.")
    parser.add_argument(
        "--target-dir",
        "--targetDir",
        required=True,
        help=(
            "Specifies the build dir where the language binding "
            "should be placed"))

    parser.add_argument(
        "--target-platform",
        help=(
            "Specifies the platform we are building for."
            "Should be the same as what platform.system() returns."))

    group = parser.add_argument_group("static binding usage")
    group.add_argument(
        "--use-static-binding",
        action="store_true",
        help=(
            "Specify the pre-baked binding can be used if "
            "swig cannot be found."))
    group.add_argument(
        "--static-binding-dir",
        default="static-binding",
        help="script-relative directory for appropriate static bindings"
    )

    # Process args.
    options = parser.parse_args(args)

    # Set logging level based on verbosity count.
    if options.debug:
        log_level = logging.DEBUG
    else:
        # See logging documentation for error levels.  We'll default
        # to showing ERROR or higher error messages.  For each -v
        # specified, we'll shift to the next lower-priority log level.
        log_level = logging.ERROR - 10 * options.verbose
        if log_level < logging.NOTSET:
            # Displays all logged messages.
            log_level = logging.NOTSET
    logging.basicConfig(level=log_level)
    logging.info("logging is using level: %d", log_level)

    return options


def find_file_in_paths(paths, exe_basename):
    """Returns the full exe path for the first path match.

    @params paths the list of directories to search for the exe_basename
    executable
    @params exe_basename the name of the file for which to search.
    e.g. "swig" or "swig.exe".

    @return the full path to the executable if found in one of the
    given paths; otherwise, returns None.
    """
    for path in paths:
        trial_exe_path = os.path.join(path, exe_basename)
        if os.path.exists(trial_exe_path):
            return os.path.normcase(trial_exe_path)
    return None


def find_swig_executable(options):
    """Finds the swig executable in the PATH or known good locations.

    :param options the command line options returned by argparse.

    Replaces options.swig_executable with the full swig executable path.
    """
    # Figure out what we're looking for.
    if platform.system() == 'Windows':
        exe_basename = "swig.exe"
        extra_dirs = []
    else:
        exe_basename = "swig"
        extra_dirs = ["/usr/local/bin"]

    # Figure out what paths to check.
    path_env = os.environ.get("PATH", None)
    if path_env is not None:
        paths_to_check = path_env.split(os.path.pathsep)
    else:
        paths_to_check = []

    # Add in the extra dirs
    paths_to_check.extend(extra_dirs)
    if len(paths_to_check) < 1:
        logging.info("failed to find swig: no paths available")
        return

    # Find the swig executable
    options.swig_executable = find_file_in_paths(paths_to_check, exe_basename)
    if not options.swig_executable or len(options.swig_executable) < 1:
        logging.info(
            "%s not found in paths %s",
            exe_basename,
            paths_to_check)
    else:
        logging.info("found swig executable: %s", options.swig_executable)


def main(args):
    """Drives the main script preparation steps.

    @param args list of command line arguments.
    """
    # Process command line arguments.
    options = process_args(args)
    logging.debug("Processed args: options=%s", options)

    if options.use_static_binding:
        # Clear the swig executable if we want to use the static bindings.
        options.swig_executable = None
        options.find_swig = False
    elif not options.swig_executable or len(options.swig_executable) == 0:
        # Ensure we have a swig executable.
        if options.find_swig:
            find_swig_executable(options)
        else:
            logging.error(
                "The --find-swig option must be specified "
                "when the swig executable location is not "
                "explicitly provided and not using the "
                "static bindings.")
            sys.exit(-12)

    # Check if the swig file exists.
    swig_path = os.path.normcase(
        os.path.join(options.src_root, "bindings", "python", "python.swig"))
    if not os.path.isfile(swig_path):
        logging.error("swig file not found at '%s'", swig_path)
        sys.exit(-3)

    # Prepare bindings for each supported language binding.
    # This will error out if it doesn't succeed.
    prepare_all_bindings(options)
    sys.exit(0)

if __name__ == "__main__":
    # Run the main driver loop.
    main(sys.argv[1:])



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetTarget()
    obj.GetByteOrder()
    obj.PutSTDIN("my data")
    obj.GetSTDOUT(6)
    obj.GetSTDERR(6)
    event = lldb.SBEvent()
    try:
        obj.ReportEventState(event, None)
    except Exception:
        pass
    obj.AppendEventStateReport(event, lldb.SBCommandReturnObject())
    error = lldb.SBError()
    obj.RemoteAttachToProcessWithID(123, error)
    obj.RemoteLaunch(None, None, None, None, None, None, 0, False, error)
    obj.GetNumThreads()
    obj.GetThreadAtIndex(0)
    obj.GetThreadByID(0)
    obj.GetSelectedThread()
    obj.SetSelectedThread(lldb.SBThread())
    obj.SetSelectedThreadByID(0)
    obj.GetState()
    obj.GetExitStatus()
    obj.GetExitDescription()
    obj.GetProcessID()
    obj.GetAddressByteSize()
    obj.Destroy()
    obj.Continue()
    obj.Stop()
    obj.Kill()
    obj.Detach()
    obj.Signal(7)
    obj.ReadMemory(0x0000FFFF, 10, error)
    obj.WriteMemory(0x0000FFFF, "hi data", error)
    obj.ReadCStringFromMemory(0x0, 128, error)
    obj.ReadUnsignedFromMemory(0xFF, 4, error)
    obj.ReadPointerFromMemory(0xFF, error)
    obj.GetBroadcaster()
    obj.GetDescription(lldb.SBStream())
    obj.LoadImage(lldb.SBFileSpec(), error)
    obj.UnloadImage(0)
    obj.Clear()
    obj.GetNumSupportedHardwareWatchpoints(error)
    for thread in obj:
        s = str(thread)
    len(obj)



"""
Test number of threads.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class CreateDuringStepTestCase(TestBase):
    @expectedFailureAll(
        oslist=["linux"],
        bugnumber="llvm.org/pr15824 thread states not properly maintained",
    )
    @expectedFailureAll(
        oslist=lldbplatformutil.getDarwinOSTriples(),
        bugnumber="llvm.org/pr15824 thread states not properly maintained, <rdar://problem/28557237>",
    )
    @expectedFailureAll(
        oslist=["freebsd"],
        bugnumber="llvm.org/pr18190 thread states not properly maintained",
    )
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24668: Breakpoints not resolved correctly",
    )
    @expectedFailureNetBSD
    def test_step_inst(self):
        """Test thread creation during step-inst handling."""
        self.build()
        self.create_during_step_base(
            "thread step-inst -m all-threads", "stop reason = instruction step"
        )

    @expectedFailureAll(
        oslist=["linux"],
        bugnumber="llvm.org/pr15824 thread states not properly maintained",
    )
    @expectedFailureAll(
        oslist=lldbplatformutil.getDarwinOSTriples(),
        bugnumber="llvm.org/pr15824 thread states not properly maintained, <rdar://problem/28557237>",
    )
    @expectedFailureAll(
        oslist=["freebsd"],
        bugnumber="llvm.org/pr18190 thread states not properly maintained",
    )
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24668: Breakpoints not resolved correctly",
    )
    @expectedFailureNetBSD
    def test_step_over(self):
        """Test thread creation during step-over handling."""
        self.build()
        self.create_during_step_base(
            "thread step-over -m all-threads", "stop reason = step over"
        )

    @expectedFailureAll(
        oslist=["linux"],
        bugnumber="llvm.org/pr15824 thread states not properly maintained",
    )
    @expectedFailureAll(
        oslist=lldbplatformutil.getDarwinOSTriples(),
        bugnumber="<rdar://problem/28574077>",
    )
    @expectedFailureAll(
        oslist=["freebsd"],
        bugnumber="llvm.org/pr18190 thread states not properly maintained",
    )
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr24668: Breakpoints not resolved correctly",
    )
    @expectedFailureNetBSD
    def test_step_in(self):
        """Test thread creation during step-in handling."""
        self.build()
        self.create_during_step_base(
            "thread step-in -m all-threads", "stop reason = step in"
        )

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break and continue.
        self.breakpoint = line_number("main.cpp", "// Set breakpoint here")
        self.continuepoint = line_number("main.cpp", "// Continue from here")

    def create_during_step_base(self, step_cmd, step_stop_reason):
        """Test thread creation while using step-in."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Get the target process
        target = self.dbg.GetSelectedTarget()

        # This should create a breakpoint in the stepping thread.
        self.bkpt = target.BreakpointCreateByLocation("main.cpp", self.breakpoint)

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        process = target.GetProcess()

        # The stop reason of the thread should be breakpoint.
        stepping_thread = lldbutil.get_one_thread_stopped_at_breakpoint(
            process, self.bkpt
        )
        self.assertTrue(stepping_thread.IsValid(), "We stopped at the right breakpoint")

        # Get the number of threads
        num_threads = process.GetNumThreads()

        # Make sure we see only two threads
        self.assertEqual(
            num_threads,
            2,
            "Number of expected threads and actual threads do not match.",
        )

        # Get the thread objects
        thread1 = process.GetThreadAtIndex(0)
        thread2 = process.GetThreadAtIndex(1)

        current_line = self.breakpoint
        # Keep stepping until we've reached our designated continue point
        while current_line != self.continuepoint:
            if stepping_thread != process.GetSelectedThread():
                process.SetSelectedThread(stepping_thread)

            self.runCmd(step_cmd)

            frame = stepping_thread.GetFrameAtIndex(0)
            current_line = frame.GetLineEntry().GetLine()

            # Make sure we're still where we thought we were
            self.assertTrue(
                current_line >= self.breakpoint,
                "Stepped to unexpected line, " + str(current_line),
            )
            self.assertTrue(
                current_line <= self.continuepoint,
                "Stepped to unexpected line, " + str(current_line),
            )

        # Update the number of threads
        num_threads = process.GetNumThreads()

        # Check to see that we increased the number of threads as expected
        self.assertEqual(
            num_threads,
            3,
            "Number of expected threads and actual threads do not match after thread exit.",
        )

        stop_reason = stepping_thread.GetStopReason()
        self.assertStopReason(
            stop_reason, lldb.eStopReasonPlanComplete, "Stopped for plan completion"
        )

        # Run to completion
        self.runCmd("process continue")

        # At this point, the inferior process should have exited.
        self.assertEqual(process.GetState(), lldb.eStateExited, PROCESS_EXITED)



import lldb


class StepWithChild:
    def __init__(self, thread_plan):
        self.thread_plan = thread_plan
        self.child_thread_plan = self.queue_child_thread_plan()

    def explains_stop(self, event):
        return False

    def should_stop(self, event):
        if not self.child_thread_plan.IsPlanComplete():
            return False

        self.thread_plan.SetPlanComplete(True)

        return True

    def should_step(self):
        return False

    def stop_description(self, stream):
        if self.child_thread_plan.IsPlanComplete():
            return self.child_thread_plan.GetDescription(stream)
        return True

    def queue_child_thread_plan(self):
        return None


class StepOut(StepWithChild):
    def __init__(self, thread_plan, dict):
        StepWithChild.__init__(self, thread_plan)

    def queue_child_thread_plan(self):
        return self.thread_plan.QueueThreadPlanForStepOut(0)


class StepScripted(StepWithChild):
    def __init__(self, thread_plan, dict):
        StepWithChild.__init__(self, thread_plan)

    def queue_child_thread_plan(self):
        return self.thread_plan.QueueThreadPlanForStepScripted("Steps.StepOut")


# This plan does a step-over until a variable changes value.
class StepUntil(StepWithChild):
    def __init__(self, thread_plan, args_data, dict):
        self.thread_plan = thread_plan
        self.frame = thread_plan.GetThread().frames[0]
        self.target = thread_plan.GetThread().GetProcess().GetTarget()
        var_entry = args_data.GetValueForKey("variable_name")

        if not var_entry.IsValid():
            print("Did not get a valid entry for variable_name")
        self.var_name = var_entry.GetStringValue(100)

        self.value = self.frame.FindVariable(self.var_name)
        if self.value.GetError().Fail():
            print("Failed to get foo value: %s" % (self.value.GetError().GetCString()))

        StepWithChild.__init__(self, thread_plan)

    def queue_child_thread_plan(self):
        le = self.frame.GetLineEntry()
        start_addr = le.GetStartAddress()
        start = start_addr.GetLoadAddress(self.target)
        end = le.GetEndAddress().GetLoadAddress(self.target)
        return self.thread_plan.QueueThreadPlanForStepOverRange(start_addr, end - start)

    def should_stop(self, event):
        if not self.child_thread_plan.IsPlanComplete():
            return False

        # If we've stepped out of this frame, stop.
        if not self.frame.IsValid():
            self.thread_plan.SetPlanComplete(True)
            return True

        if not self.value.IsValid():
            self.thread_plan.SetPlanComplete(True)
            return True

        if not self.value.GetValueDidChange():
            self.child_thread_plan = self.queue_child_thread_plan()
            return False
        else:
            self.thread_plan.SetPlanComplete(True)
            return True

    def stop_description(self, stream):
        stream.Print(f"Stepped until {self.var_name} changed.")


# This plan does nothing, but sets stop_mode to the
# value of GetStopOthers for this plan.
class StepReportsStopOthers:
    stop_mode_dict = {}

    def __init__(self, thread_plan, args_data, dict):
        self.thread_plan = thread_plan
        self.key = str(args_data.GetValueForKey("token").GetUnsignedIntegerValue(1000))

    def should_stop(self, event):
        self.thread_plan.SetPlanComplete(True)
        StepReportsStopOthers.stop_mode_dict[
            self.key
        ] = self.thread_plan.GetStopOthers()
        return True

    def should_step(self):
        return True

    def explains_stop(self, event):
        return True



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestWatchpointCount(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(
        oslist=["freebsd", "linux"],
        archs=["arm", "aarch64"],
        bugnumber="llvm.org/pr26031",
    )
    def test_watchpoint_count(self):
        self.build()
        (_, process, thread, _) = lldbutil.run_to_source_breakpoint(
            self, "patatino", lldb.SBFileSpec("main.c")
        )
        frame = thread.GetFrameAtIndex(0)
        first_var = frame.FindVariable("x1")
        second_var = frame.FindVariable("x2")

        error = lldb.SBError()
        first_watch = first_var.Watch(True, False, True, error)
        if not error.Success():
            self.fail("Failed to make watchpoint for x1: %s" % (error.GetCString()))

        second_watch = second_var.Watch(True, False, True, error)
        if not error.Success():
            self.fail("Failed to make watchpoint for x2: %s" % (error.GetCString()))
        process.Continue()

        stop_reason = thread.GetStopReason()
        self.assertStopReason(
            stop_reason, lldb.eStopReasonWatchpoint, "watchpoint for x1 not hit"
        )
        stop_reason_descr = thread.GetStopDescription(256)
        self.assertEqual(stop_reason_descr, "watchpoint 1")

        process.Continue()
        stop_reason = thread.GetStopReason()
        self.assertStopReason(
            stop_reason, lldb.eStopReasonWatchpoint, "watchpoint for x2 not hit"
        )
        stop_reason_descr = thread.GetStopDescription(256)
        self.assertEqual(stop_reason_descr, "watchpoint 2")



# TestSwiftStdlibSet.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Tests that we properly vend synthetic children for Swift.Set
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftStdlibSet(TestBase):
    @swiftTest
    def test_swift_stdlib_set(self):
        """Tests that we properly vend synthetic children for Swift.Set"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'break here', lldb.SBFileSpec('main.swift'))
        self.expect(
            "frame variable",
            ordered=False,
            substrs=[
                ' = 5',
                ' = 2',
                ' = 3',
                ' = 1',
                ' = 4'])





"""
Test lldb watchpoint that uses '-s size' to watch a pointed location with size.
"""


import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class HelloWatchLocationTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Our simple source filename.
        self.source = "main.cpp"
        # Find the line number to break inside main().
        self.line = line_number(self.source, "// Set break point at this line.")
        # This is for verifying that watch location works.
        self.violating_func = "do_bad_thing_with_location"
        # Build dictionary to have unique executable names for each test
        # method.
        self.exe_name = self.testMethodName
        self.d = {"CXX_SOURCES": self.source, "EXE": self.exe_name}

    # on arm64 targets, lldb has incorrect hit-count / ignore-counts
    # for watchpoints when they are hit with multiple threads at
    # the same time.  Tracked as llvm.org/pr49433
    # or rdar://93863107 inside Apple.
    def affected_by_radar_93863107(self):
        return (
            self.getArchitecture() in ["arm64", "arm64e"]
        ) and self.platformIsDarwin()

    # Most of the MIPS boards provide only one H/W watchpoints, and S/W
    # watchpoints are not supported yet
    @expectedFailureAll(triple=re.compile("^mips"))
    # SystemZ and PowerPC also currently supports only one H/W watchpoint
    @expectedFailureAll(archs=["powerpc64le", "s390x"])
    @skipIfWindows  # This test is flaky on Windows
    def test_hello_watchlocation(self):
        """Test watching a location with '-s size' option."""
        self.build(dictionary=self.d)
        self.setTearDownCleanup(dictionary=self.d)
        exe = self.getBuildArtifact(self.exe_name)
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Add a breakpoint to set a watchpoint when stopped on the breakpoint.
        lldbutil.run_break_set_by_file_and_line(
            self, None, self.line, num_expected_locations=1, loc_exact=False
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # We should be stopped again due to the breakpoint.
        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Now let's set a write-type watchpoint pointed to by 'g_char_ptr'.
        self.expect(
            "watchpoint set expression -w write -s 1 -- g_char_ptr",
            WATCHPOINT_CREATED,
            substrs=["Watchpoint created", "size = 1", "type = w"],
        )
        # Get a hold of the watchpoint id just created, it is used later on to
        # match the watchpoint id which is expected to be fired.
        match = re.match(
            "Watchpoint created: Watchpoint (.*):", self.res.GetOutput().splitlines()[0]
        )
        if match:
            expected_wp_id = int(match.group(1), 0)
        else:
            self.fail("Grokking watchpoint id faailed!")

        self.runCmd("expr unsigned val = *g_char_ptr; val")
        self.expect(self.res.GetOutput().splitlines()[0], exe=False, endstr=" = 0")

        self.runCmd("watchpoint set expression -w write -s 4 -- &threads[0]")

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should be 0 initially.
        self.expect("watchpoint list -v", substrs=["hit_count = 0"])

        self.runCmd("process continue")

        # We should be stopped again due to the watchpoint (write type), but
        # only once.  The stop reason of the thread should be watchpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_WATCHPOINT,
            substrs=["stopped", "stop reason = watchpoint %d" % expected_wp_id],
        )

        # Switch to the thread stopped due to watchpoint and issue some
        # commands.
        self.switch_to_thread_with_stop_reason(lldb.eStopReasonWatchpoint)
        self.runCmd("thread backtrace")
        self.expect("frame info", substrs=[self.violating_func])

        # Use the '-v' option to do verbose listing of the watchpoint.
        # The hit count should now be the same as the number of threads that
        # stopped on a watchpoint.
        threads = lldbutil.get_stopped_threads(
            self.process(), lldb.eStopReasonWatchpoint
        )

        if not self.affected_by_radar_93863107():
            self.expect("watchpoint list -v", substrs=["hit_count = %d" % len(threads)])

        self.runCmd("thread backtrace all")



"""Test that we are able to evaluate expressions when the inferior is blocked in a syscall"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprSyscallTestCase(TestBase):
    @expectedFailureAll(
        oslist=["windows"],
        bugnumber="llvm.org/pr21765, getpid() does not exist on Windows",
    )
    @expectedFailureNetBSD
    def test_setpgid(self):
        self.build()
        self.expr_syscall()

    def expr_syscall(self):
        # Create a target by the debugger.
        target = self.createTestTarget()

        listener = lldb.SBListener("my listener")

        # launch the inferior and don't wait for it to stop
        self.dbg.SetAsync(True)
        error = lldb.SBError()
        flags = target.GetLaunchInfo().GetLaunchFlags()
        process = target.Launch(
            listener,
            None,  # argv
            None,  # envp
            None,  # stdin_path
            None,  # stdout_path
            None,  # stderr_path
            None,  # working directory
            flags,  # launch flags
            False,  # Stop at entry
            error,
        )  # error

        self.assertTrue(process and process.IsValid(), PROCESS_IS_VALID)

        event = lldb.SBEvent()

        # Give the child enough time to reach the syscall,
        # while clearing out all the pending events.
        # The last WaitForEvent call will time out after 2 seconds.
        while listener.WaitForEvent(2, event):
            pass

        # now the process should be running (blocked in the syscall)
        self.assertEqual(process.GetState(), lldb.eStateRunning, "Process is running")

        # send the process a signal
        process.SendAsyncInterrupt()
        while listener.WaitForEvent(2, event):
            pass

        # as a result the process should stop
        # in all likelihood we have stopped in the middle of the sleep()
        # syscall
        self.assertEqual(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        thread = process.GetSelectedThread()

        # try evaluating a couple of expressions in this state
        self.expect_expr("release_flag = 1", result_value="1")
        self.expect_expr("(int)getpid()", result_value=str(process.GetProcessID()))

        # and run the process to completion
        process.Continue()

        # process all events
        while listener.WaitForEvent(10, event):
            new_state = lldb.SBProcess.GetStateFromEvent(event)
            if new_state == lldb.eStateExited:
                break

        self.assertState(process.GetState(), lldb.eStateExited)
        self.assertEqual(process.GetExitStatus(), 0)



import lldb
from intelpt_testcase import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
from lldbsuite.test.decorators import *


class TestTraceStartStop(TraceIntelPTTestCaseBase):
    def expectGenericHelpMessageForStartCommand(self):
        self.expect(
            "help thread trace start",
            substrs=["Syntax: thread trace start [<trace-options>]"],
        )

    @testSBAPIAndCommands
    def testStartStopSessionFileThreads(self):
        # it should fail for processes from json session files
        self.expect(
            "trace load -v "
            + os.path.join(self.getSourceDir(), "intelpt-trace", "trace.json")
        )

        # the help command should be the generic one, as it's not a live process
        self.expectGenericHelpMessageForStartCommand()

        self.traceStartThread(error=True)

        self.traceStopThread(error=True)

    @testSBAPIAndCommands
    def testStartWithNoProcess(self):
        self.traceStartThread(error=True)

    @testSBAPIAndCommands
    def testStartSessionWithWrongSize(self):
        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")

        self.traceStartThread(
            error=True,
            iptTraceSize=2000,
            substrs=["The intel pt trace size must be a power of 2", "It was 2000"],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize=5000,
            substrs=["The intel pt trace size must be a power of 2", "It was 5000"],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize=0,
            substrs=["The intel pt trace size must be a power of 2", "It was 0"],
        )

        self.traceStartThread(iptTraceSize=1048576)

    @testSBAPIAndCommands
    def testStartSessionWithSizeDeclarationInUnits(self):
        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")

        self.traceStartThread(
            error=True,
            iptTraceSize="abc",
            substrs=["invalid bytes expression for 'abc'"],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="123.12",
            substrs=["invalid bytes expression for '123.12'"],
        )

        self.traceStartThread(
            error=True, iptTraceSize='""', substrs=["invalid bytes expression for ''"]
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="2000B",
            substrs=[
                "The intel pt trace size must be a power of 2 greater than or equal to 4096 (2^12) bytes. It was 2000"
            ],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="3MB",
            substrs=[
                "The intel pt trace size must be a power of 2 greater than or equal to 4096 (2^12) bytes. It was 3145728"
            ],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="3MiB",
            substrs=[
                "The intel pt trace size must be a power of 2 greater than or equal to 4096 (2^12) bytes. It was 3145728"
            ],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="3mib",
            substrs=[
                "The intel pt trace size must be a power of 2 greater than or equal to 4096 (2^12) bytes. It was 3145728"
            ],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="3M",
            substrs=[
                "The intel pt trace size must be a power of 2 greater than or equal to 4096 (2^12) bytes. It was 3145728"
            ],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="3KB",
            substrs=[
                "The intel pt trace size must be a power of 2 greater than or equal to 4096 (2^12) bytes. It was 3072"
            ],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="3KiB",
            substrs=[
                "The intel pt trace size must be a power of 2 greater than or equal to 4096 (2^12) bytes. It was 3072"
            ],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="3K",
            substrs=[
                "The intel pt trace size must be a power of 2 greater than or equal to 4096 (2^12) bytes. It was 3072"
            ],
        )

        self.traceStartThread(
            error=True,
            iptTraceSize="3MS",
            substrs=["invalid bytes expression for '3MS'"],
        )

        self.traceStartThread(iptTraceSize="1048576")

    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testSBAPIHelp(self):
        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")

        help = self.getTraceOrCreate().GetStartConfigurationHelp()
        self.assertIn("iptTraceSize", help)
        self.assertIn("processBufferSizeLimit", help)

    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testStoppingAThread(self):
        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")
        self.expect("r")
        self.expect("thread trace start")
        self.expect("n")
        self.expect(
            "thread trace dump instructions",
            substrs=[
                """0x0000000000400511    movl   $0x0, -0x4(%rbp)
    no more data"""
            ],
        )
        # process stopping should stop the thread
        self.expect("process trace stop")
        self.expect("n")
        self.expect(
            "thread trace dump instructions", substrs=["not traced"], error=True
        )

    @skipIf(oslist=no_match(["linux"]), archs=no_match(["i386", "x86_64"]))
    def testStartStopLiveThreads(self):
        # The help command should be the generic one if there's no process running
        self.expectGenericHelpMessageForStartCommand()

        self.expect(
            "thread trace start", error=True, substrs=["error: Process not available"]
        )

        self.expect(
            "file " + os.path.join(self.getSourceDir(), "intelpt-trace", "a.out")
        )
        self.expect("b main")

        self.expect(
            "thread trace start", error=True, substrs=["error: Process not available"]
        )

        # The help command should be the generic one if there's still no process running
        self.expectGenericHelpMessageForStartCommand()

        self.expect("r")

        # This fails because "trace start" hasn't been called yet
        self.expect(
            "thread trace stop",
            error=True,
            substrs=["error: Process is not being traced"],
        )

        # the help command should be the intel-pt one now
        self.expect(
            "help thread trace start",
            substrs=[
                "Start tracing one or more threads with intel-pt.",
                "Syntax: thread trace start [<thread-index> <thread-index> ...] [<intel-pt-options>]",
            ],
        )

        # We start tracing with a small buffer size
        self.expect("thread trace start 1 --size 4096")

        # We fail if we try to trace again
        self.expect(
            "thread trace start",
            error=True,
            substrs=["error: Thread ", "already traced"],
        )

        # We can reconstruct the single instruction executed in the first line
        self.expect("n")
        self.expect(
            "thread trace dump instructions -f",
            patterns=[
                f"""thread #1: tid = .*
  a.out`main \+ 4 at main.cpp:2
    2: {ADDRESS_REGEX}    movl"""
            ],
        )

        # We can reconstruct the instructions up to the second line
        self.expect("n")
        self.expect(
            "thread trace dump instructions -f",
            patterns=[
                f"""thread #1: tid = .*
  a.out`main \+ 4 at main.cpp:2
    2: {ADDRESS_REGEX}    movl .*
  a.out`main \+ 11 at main.cpp:4
    4: {ADDRESS_REGEX}    movl .*
    6: {ADDRESS_REGEX}    jmp  .* ; <\+28> at main.cpp:4
    8: {ADDRESS_REGEX}    cmpl .*
    10: {ADDRESS_REGEX}    jle  .* ; <\+20> at main.cpp:5"""
            ],
        )

        self.expect(
            "thread trace dump instructions",
            patterns=[
                f"""thread #1: tid = .*
  a.out`main \+ 32 at main.cpp:4
    10: {ADDRESS_REGEX}    jle  .* ; <\+20> at main.cpp:5
    8: {ADDRESS_REGEX}    cmpl .*
    6: {ADDRESS_REGEX}    jmp  .* ; <\+28> at main.cpp:4
    4: {ADDRESS_REGEX}    movl .*
  a.out`main \+ 4 at main.cpp:2
    2: {ADDRESS_REGEX}    movl .* """
            ],
        )

        # We stop tracing
        self.expect("thread trace stop")

        # We can't stop twice
        self.expect(
            "thread trace stop",
            error=True,
            substrs=["error: Thread ", "not currently traced"],
        )

        # We trace again from scratch, this time letting LLDB to pick the current
        # thread
        self.expect("thread trace start")
        self.expect("n")
        self.expect(
            "thread trace dump instructions -f",
            patterns=[
                f"""thread #1: tid = .*
  a.out`main \+ 20 at main.cpp:5
    2: {ADDRESS_REGEX}    xorl"""
            ],
        )

        self.expect(
            "thread trace dump instructions",
            patterns=[
                f"""thread #1: tid = .*
  a.out`main \+ 20 at main.cpp:5
    2: {ADDRESS_REGEX}    xorl"""
            ],
        )

        self.expect("c")
        # Now the process has finished, so the commands should fail
        self.expect(
            "thread trace start",
            error=True,
            substrs=["error: Process must be launched"],
        )

        self.expect(
            "thread trace stop", error=True, substrs=["error: Process must be launched"]
        )

        # We should be able to trace the program if we relaunch it
        # For this, we'll trace starting at a different point in the new
        # process.
        self.expect("breakpoint disable")
        self.expect("b main.cpp:4")
        self.expect("r")
        self.expect("thread trace start")
        # We can reconstruct the single instruction executed in the first line
        self.expect("si")
        self.expect(
            "thread trace dump instructions -c 1",
            patterns=[
                f"""thread #1: tid = .*
  a.out`main \+ 11 at main.cpp:4"""
            ],
        )



import glob
import json
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os
import time


class DebugIndexCacheTestcase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Set the lldb module cache directory to a directory inside the build
        # artifacts directory so no other tests are interfered with.
        self.cache_dir = os.path.join(self.getBuildDir(), "lldb-module-cache")

    def get_module_cache_files(self, basename):
        module_cache_glob = os.path.join(
            self.cache_dir, "llvmcache-*%s*dwarf-index*" % (basename)
        )
        return glob.glob(module_cache_glob)

    def get_stats(self, log_path=None):
        """
        Get the output of the "statistics dump" and return the JSON as a
        python dictionary.
        """
        # If log_path is set, open the path and emit the output of the command
        # for debugging purposes.
        if log_path is not None:
            f = open(log_path, "w")
        else:
            f = None
        return_obj = lldb.SBCommandReturnObject()
        command = "statistics dump "
        if f:
            f.write("(lldb) %s\n" % (command))
        self.ci.HandleCommand(command, return_obj, False)
        metrics_json = return_obj.GetOutput()
        if f:
            f.write(metrics_json)
        return json.loads(metrics_json)

    def enable_lldb_index_cache(self):
        self.runCmd(
            'settings set symbols.lldb-index-cache-path "%s"' % (self.cache_dir)
        )
        self.runCmd("settings set symbols.enable-lldb-index-cache true")

    @no_debug_info_test
    def test_with_caching_enabled(self):
        """
        Test module cache functionality for debug info index caching.

        We test that a debug info index file is created for the debug
        information when caching is enabled with a file that contains
        at least one of each kind of DIE in ManualDWARFIndex::IndexSet.

        The input file has DWARF that will fill in every member of the
        ManualDWARFIndex::IndexSet class to ensure we can encode all of the
        required information.

        With caching enabled, we also verify that the appropriate statistics
        specify that the cache file was saved to the cache.
        """
        self.enable_lldb_index_cache()
        src_dir = self.getSourceDir()
        yaml_path = os.path.join(src_dir, "exe.yaml")
        yaml_base, ext = os.path.splitext(yaml_path)
        obj_path = self.getBuildArtifact("main.o")
        self.yaml2obj(yaml_path, obj_path)

        # Create a target with the object file we just created from YAML
        target = self.dbg.CreateTarget(obj_path)
        self.assertTrue(target, VALID_TARGET)

        debug_index_cache_files = self.get_module_cache_files("main.o")
        self.assertEqual(
            len(debug_index_cache_files),
            1,
            "make sure there is one file in the module cache directory (%s) for main.o that is a debug info cache"
            % (self.cache_dir),
        )

        # Verify that the module statistics have the information that specifies
        # if we loaded or saved the debug index and symtab to the cache
        stats = self.get_stats()
        module_stats = stats["modules"][0]
        self.assertFalse(module_stats["debugInfoIndexLoadedFromCache"])
        self.assertTrue(module_stats["debugInfoIndexSavedToCache"])
        self.assertFalse(module_stats["symbolTableLoadedFromCache"])
        self.assertTrue(module_stats["symbolTableSavedToCache"])
        # Verify the top level stats track how many things were loaded or saved
        # to the cache.
        self.assertEqual(stats["totalDebugInfoIndexLoadedFromCache"], 0)
        self.assertEqual(stats["totalDebugInfoIndexSavedToCache"], 1)
        self.assertEqual(stats["totalSymbolTablesLoadedFromCache"], 0)
        self.assertEqual(stats["totalSymbolTablesSavedToCache"], 1)

    @no_debug_info_test
    def test_with_caching_disabled(self):
        """
        Test module cache functionality for debug info index caching.

        We test that a debug info index file is not created for the debug
        information when caching is disabled with a file that contains
        at least one of each kind of DIE in ManualDWARFIndex::IndexSet.

        The input file has DWARF that will fill in every member of the
        ManualDWARFIndex::IndexSet class to ensure we can encode all of the
        required information.

        With caching disabled, we also verify that the appropriate
        statistics specify that the cache file was not saved to the cache.
        """
        src_dir = self.getSourceDir()
        yaml_path = os.path.join(src_dir, "exe.yaml")
        yaml_base, ext = os.path.splitext(yaml_path)
        obj_path = self.getBuildArtifact("main.o")
        self.yaml2obj(yaml_path, obj_path)

        # Create a target with the object file we just created from YAML
        target = self.dbg.CreateTarget(obj_path)
        self.assertTrue(target, VALID_TARGET)

        debug_index_cache_files = self.get_module_cache_files("main.o")
        self.assertEqual(
            len(debug_index_cache_files),
            0,
            "make sure there is no file in the module cache directory (%s) for main.o that is a debug info cache"
            % (self.cache_dir),
        )

        # Verify that the module statistics have the information that specifies
        # if we loaded or saved the debug index and symtab to the cache
        stats = self.get_stats()
        module_stats = stats["modules"][0]
        self.assertFalse(module_stats["debugInfoIndexLoadedFromCache"])
        self.assertFalse(module_stats["debugInfoIndexSavedToCache"])
        self.assertFalse(module_stats["symbolTableLoadedFromCache"])
        self.assertFalse(module_stats["symbolTableSavedToCache"])
        # Verify the top level stats track how many things were loaded or saved
        # to the cache.
        self.assertEqual(stats["totalDebugInfoIndexLoadedFromCache"], 0)
        self.assertEqual(stats["totalDebugInfoIndexSavedToCache"], 0)
        self.assertEqual(stats["totalSymbolTablesLoadedFromCache"], 0)
        self.assertEqual(stats["totalSymbolTablesSavedToCache"], 0)



"""
Test dwim-print with variables, variable paths, and expressions.
"""

import re
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil


class TestCase(TestBase):
    def _run_cmd(self, cmd: str) -> str:
        """Run the given lldb command and return its output."""
        result = lldb.SBCommandReturnObject()
        self.ci.HandleCommand(cmd, result)
        return result.GetOutput().rstrip()

    VAR_IDENT = re.compile(r"(?:\$\d+|\w+) = ")

    def _strip_result_var(self, string: str) -> str:
        """
        Strip (persistent) result variables (ex '$0 = ', or 'someVar = ', etc).

        This allows for using the output of `expression`/`frame variable`, to
        compare it to `dwim-print` output, which disables result variables.
        """
        return self.VAR_IDENT.subn("", string, 1)[0]

    def _expect_cmd(
        self,
        dwim_cmd: str,
        actual_cmd: str,
    ) -> None:
        """Run dwim-print and verify the output against the expected command."""
        # Resolve the dwim-print command to either `expression` or `frame variable`.
        substitute_cmd = dwim_cmd.replace("dwim-print", actual_cmd, 1)
        interp = self.dbg.GetCommandInterpreter()
        result = lldb.SBCommandReturnObject()
        interp.ResolveCommand(substitute_cmd, result)
        self.assertTrue(result.Succeeded(), result.GetError())

        resolved_cmd = result.GetOutput()
        if actual_cmd == "frame variable":
            resolved_cmd = resolved_cmd.replace(" -- ", " ", 1)

        resolved_cmd_output = self._run_cmd(resolved_cmd)
        dwim_cmd_output = self._strip_result_var(resolved_cmd_output)

        # Verify dwim-print chose the expected command.
        self.runCmd("settings set dwim-print-verbosity full")

        self.expect(
            dwim_cmd,
            substrs=[
                f"note: ran `{resolved_cmd}`",
                dwim_cmd_output,
            ],
        )

    def test_variables(self):
        """Test dwim-print with variables."""
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        vars = ("argc", "argv")
        for var in vars:
            self._expect_cmd(f"dwim-print {var}", "frame variable")

    def test_variable_paths(self):
        """Test dwim-print with variable path expressions."""
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        exprs = ("&argc", "*argv", "argv[0]")
        for expr in exprs:
            self._expect_cmd(f"dwim-print {expr}", "expression")

    def test_expressions(self):
        """Test dwim-print with expressions."""
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        exprs = ("argc + 1", "(void)argc", "(int)abs(argc)")
        for expr in exprs:
            self._expect_cmd(f"dwim-print {expr}", "expression")

    def test_dummy_target_expressions(self):
        """Test dwim-print's ability to evaluate expressions without a target."""
        self._expect_cmd("dwim-print 1 + 2", "expression")

    def test_gdb_format(self):
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        self._expect_cmd(f"dwim-print/x argc", "frame variable")
        self._expect_cmd(f"dwim-print/x argc + 1", "expression")

    def test_format_flags(self):
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        self._expect_cmd(f"dwim-print -fx -- argc", "frame variable")
        self._expect_cmd(f"dwim-print -fx -- argc + 1", "expression")

    def test_display_flags(self):
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        self._expect_cmd(f"dwim-print -T -- argc", "frame variable")
        self._expect_cmd(f"dwim-print -T -- argc + 1", "expression")

    def test_expression_language(self):
        """Test that the language flag doesn't affect the choice of command."""
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        self._expect_cmd(f"dwim-print -l c++ -- argc", "frame variable")
        self._expect_cmd(f"dwim-print -l c++ -- argc + 1", "expression")

    def test_empty_expression(self):
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        error_msg = "error: 'dwim-print' takes a variable or expression"
        self.expect(f"dwim-print", error=True, startstr=error_msg)
        self.expect(f"dwim-print -- ", error=True, startstr=error_msg)

    def test_nested_values(self):
        """Test dwim-print with nested values (structs, etc)."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        self.runCmd("settings set auto-one-line-summaries false")
        self._expect_cmd(f"dwim-print s", "frame variable")
        self._expect_cmd(f"dwim-print (struct Structure)s", "expression")

    def test_summary_strings(self):
        """Test dwim-print with nested values (structs, etc)."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        self.runCmd("settings set auto-one-line-summaries false")
        self.runCmd("type summary add -e -s 'stub summary' Structure")
        self._expect_cmd(f"dwim-print s", "frame variable")
        self._expect_cmd(f"dwim-print (struct Structure)s", "expression")

    def test_void_result(self):
        """Test dwim-print does not surface an error message for void expressions."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        self.expect("dwim-print (void)15", matching=False, patterns=["(?i)error"])

    def test_preserves_persistent_variables(self):
        """Test dwim-print does not delete persistent variables."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        self.expect("dwim-print int $i = 15")
        # Run the same expression twice and verify success. This ensures the
        # first command does not delete the persistent variable.
        for _ in range(2):
            self.expect("dwim-print $i", startstr="(int) 15")

    def test_missing_type(self):
        """The expected output of po opaque is its address (no error)"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.c")
        )
        self.expect("dwim-print -O -- opaque", substrs=['0x'])



# TestSwiftLetInt.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2016 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
"""
Test that a 'let' Int is formatted properly
"""
import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil
import os
import unittest2


class TestSwiftLetIntSupport(TestBase):
    @swiftTest
    def test_swift_let_int(self):
        """Test that a 'let' Int is formatted properly"""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, 'Set breakpoint here', lldb.SBFileSpec('main.swift'))

        let = self.frame().FindVariable("x")
        var = self.frame().FindVariable("y")
        lldbutil.check_variable(self, let, False, value="10")
        lldbutil.check_variable(self, var, False, value="10")

        get_arguments = False
        get_locals = True
        get_statics = False
        get_in_scope_only = True
        local_vars = self.frame().GetVariables(get_arguments, get_locals,
                                             get_statics, get_in_scope_only)
        self.assertTrue(local_vars.GetFirstValueByName("x").IsValid())
        self.assertTrue(local_vars.GetFirstValueByName("y").IsValid())
        self.assertTrue(not local_vars.GetFirstValueByName("z").IsValid())




import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test.gdbclientutils import *
from lldbsuite.test.lldbgdbclient import GDBRemoteTestBase


class TestJLink6Armv7RegisterDefinition(GDBRemoteTestBase):
    @skipIfXmlSupportMissing
    @skipIfRemote
    def test(self):
        """
        Test lldb's parsing of SEGGER J-Link v6.54 register
        definition for a Cortex M-4 dev board, and the fact
        that the J-Link only supports g/G for reading/writing
        register AND the J-Link v6.54 doesn't provide anything
        but the general purpose registers."""

        class MyResponder(MockGDBServerResponder):
            def qXferRead(self, obj, annex, offset, length):
                if annex == "target.xml":
                    return (
                        """<?xml version="1.0"?>
<!-- Copyright (C) 2008 Free Software Foundation, Inc.

     Copying and distribution of this file, with or without modification,
     are permitted in any medium without royalty provided the copyright
     notice and this notice are preserved.  -->

<!DOCTYPE feature SYSTEM "gdb-target.dtd">
<target version="1.0">
  <architecture>arm</architecture>
  <feature name="org.gnu.gdb.arm.m-profile">
    <reg name="r0" bitsize="32" regnum="0" type="uint32" group="general"/>
    <reg name="r1" bitsize="32" regnum="1" type="uint32" group="general"/>
    <reg name="r2" bitsize="32" regnum="2" type="uint32" group="general"/>
    <reg name="r3" bitsize="32" regnum="3" type="uint32" group="general"/>
    <reg name="r4" bitsize="32" regnum="4" type="uint32" group="general"/>
    <reg name="r5" bitsize="32" regnum="5" type="uint32" group="general"/>
    <reg name="r6" bitsize="32" regnum="6" type="uint32" group="general"/>
    <reg name="r7" bitsize="32" regnum="7" type="uint32" group="general"/>
    <reg name="r8" bitsize="32" regnum="8" type="uint32" group="general"/>
    <reg name="r9" bitsize="32" regnum="9" type="uint32" group="general"/>
    <reg name="r10" bitsize="32" regnum="10" type="uint32" group="general"/>
    <reg name="r11" bitsize="32" regnum="11" type="uint32" group="general"/>
    <reg name="r12" bitsize="32" regnum="12" type="uint32" group="general"/>
    <reg name="sp" bitsize="32" regnum="13" type="data_ptr" group="general"/>
    <reg name="lr" bitsize="32" regnum="14" type="uint32" group="general"/>
    <reg name="pc" bitsize="32" regnum="15" type="code_ptr" group="general"/>
    <reg name="xpsr" bitsize="32" regnum="25" type="uint32" group="general"/>
  </feature>
  <feature name="org.gnu.gdb.arm.m-system">
    <reg name="msp" bitsize="32" regnum="26" type="uint32" group="general"/>
    <reg name="psp" bitsize="32" regnum="27" type="uint32" group="general"/>
    <reg name="primask" bitsize="32" regnum="28" type="uint32" group="general"/>
    <reg name="basepri" bitsize="32" regnum="29" type="uint32" group="general"/>
    <reg name="faultmask" bitsize="32" regnum="30" type="uint32" group="general"/>
    <reg name="control" bitsize="32" regnum="31" type="uint32" group="general"/>
  </feature>
  <feature name="org.gnu.gdb.arm.m-float">
    <reg name="fpscr" bitsize="32" regnum="32" type="uint32" group="float"/>
    <reg name="s0" bitsize="32" regnum="33" type="float" group="float"/>
    <reg name="s1" bitsize="32" regnum="34" type="float" group="float"/>
    <reg name="s2" bitsize="32" regnum="35" type="float" group="float"/>
    <reg name="s3" bitsize="32" regnum="36" type="float" group="float"/>
    <reg name="s4" bitsize="32" regnum="37" type="float" group="float"/>
    <reg name="s5" bitsize="32" regnum="38" type="float" group="float"/>
    <reg name="s6" bitsize="32" regnum="39" type="float" group="float"/>
    <reg name="s7" bitsize="32" regnum="40" type="float" group="float"/>
    <reg name="s8" bitsize="32" regnum="41" type="float" group="float"/>
    <reg name="s9" bitsize="32" regnum="42" type="float" group="float"/>
    <reg name="s10" bitsize="32" regnum="43" type="float" group="float"/>
    <reg name="s11" bitsize="32" regnum="44" type="float" group="float"/>
    <reg name="s12" bitsize="32" regnum="45" type="float" group="float"/>
    <reg name="s13" bitsize="32" regnum="46" type="float" group="float"/>
    <reg name="s14" bitsize="32" regnum="47" type="float" group="float"/>
    <reg name="s15" bitsize="32" regnum="48" type="float" group="float"/>
    <reg name="s16" bitsize="32" regnum="49" type="float" group="float"/>
    <reg name="s17" bitsize="32" regnum="50" type="float" group="float"/>
    <reg name="s18" bitsize="32" regnum="51" type="float" group="float"/>
    <reg name="s19" bitsize="32" regnum="52" type="float" group="float"/>
    <reg name="s20" bitsize="32" regnum="53" type="float" group="float"/>
    <reg name="s21" bitsize="32" regnum="54" type="float" group="float"/>
    <reg name="s22" bitsize="32" regnum="55" type="float" group="float"/>
    <reg name="s23" bitsize="32" regnum="56" type="float" group="float"/>
    <reg name="s24" bitsize="32" regnum="57" type="float" group="float"/>
    <reg name="s25" bitsize="32" regnum="58" type="float" group="float"/>
    <reg name="s26" bitsize="32" regnum="59" type="float" group="float"/>
    <reg name="s27" bitsize="32" regnum="60" type="float" group="float"/>
    <reg name="s28" bitsize="32" regnum="61" type="float" group="float"/>
    <reg name="s29" bitsize="32" regnum="62" type="float" group="float"/>
    <reg name="s30" bitsize="32" regnum="63" type="float" group="float"/>
    <reg name="s31" bitsize="32" regnum="64" type="float" group="float"/>
    <reg name="d0" bitsize="64" regnum="65" type="ieee_double" group="float"/>
    <reg name="d1" bitsize="64" regnum="66" type="ieee_double" group="float"/>
    <reg name="d2" bitsize="64" regnum="67" type="ieee_double" group="float"/>
    <reg name="d3" bitsize="64" regnum="68" type="ieee_double" group="float"/>
    <reg name="d4" bitsize="64" regnum="69" type="ieee_double" group="float"/>
    <reg name="d5" bitsize="64" regnum="70" type="ieee_double" group="float"/>
    <reg name="d6" bitsize="64" regnum="71" type="ieee_double" group="float"/>
    <reg name="d7" bitsize="64" regnum="72" type="ieee_double" group="float"/>
    <reg name="d8" bitsize="64" regnum="73" type="ieee_double" group="float"/>
    <reg name="d9" bitsize="64" regnum="74" type="ieee_double" group="float"/>
    <reg name="d10" bitsize="64" regnum="75" type="ieee_double" group="float"/>
    <reg name="d11" bitsize="64" regnum="76" type="ieee_double" group="float"/>
    <reg name="d12" bitsize="64" regnum="77" type="ieee_double" group="float"/>
    <reg name="d13" bitsize="64" regnum="78" type="ieee_double" group="float"/>
    <reg name="d14" bitsize="64" regnum="79" type="ieee_double" group="float"/>
    <reg name="d15" bitsize="64" regnum="80" type="ieee_double" group="float"/>
  </feature>
</target>""",
                        False,
                    )
                else:
                    return None, False

            def readRegister(self, regnum):
                return "E01"

            # Initial r1 bytes, in little-endian order
            r1_bytes = "01000000"

            ## readRegisters only provides reg values up through xpsr (0x61000000)
            ## it doesn't send up any of the exception registers or floating point
            ## registers that the above register xml describes.
            def readRegisters(self):
                return (
                    "00000000"
                    + self.r1_bytes
                    + "010000000100000001000000000000008c080020a872012000000000a0790120000000008065012041ad0008a0720120692a00089e26000800000061"
                )

            ## the J-Link accepts a register write packet with just the GPRs
            ## defined.
            def writeRegisters(self, registers_hex):
                # Check that lldb returns the full 704 hex-byte register context,
                # or the 136 hex-byte general purpose register reg ctx.
                if len(registers_hex) != 704 and len(register_hex) != 136:
                    return "E06"
                if registers_hex.startswith(
                    "0000000044332211010000000100000001000000000000008c080020a872012000000000a0790120000000008065012041ad0008a0720120692a00089e26000800000061"
                ):
                    self.r1_bytes = "44332211"
                    return "OK"
                else:
                    return "E07"

            def haltReason(self):
                return "S05"

            def qfThreadInfo(self):
                return "mdead"

            def qC(self):
                return ""

            def qSupported(self, client_supported):
                return "PacketSize=4000;qXfer:memory-map:read-;QStartNoAckMode+;hwbreak+;qXfer:features:read+"

            def QThreadSuffixSupported(self):
                return "OK"

            def QListThreadsInStopReply(self):
                return "OK"

        self.server.responder = MyResponder()
        if self.TraceOn():
            self.runCmd("log enable gdb-remote packets")
            self.addTearDownHook(lambda: self.runCmd("log disable gdb-remote packets"))

        self.dbg.SetDefaultArchitecture("armv7em")
        target = self.dbg.CreateTargetWithFileAndArch(None, None)

        process = self.connect(target)

        if self.TraceOn():
            interp = self.dbg.GetCommandInterpreter()
            result = lldb.SBCommandReturnObject()
            interp.HandleCommand("target list", result)
            print(result.GetOutput())

        r1_valobj = process.GetThreadAtIndex(0).GetFrameAtIndex(0).FindRegister("r1")
        self.assertEqual(r1_valobj.GetValueAsUnsigned(), 1)

        pc_valobj = process.GetThreadAtIndex(0).GetFrameAtIndex(0).FindRegister("pc")
        self.assertEqual(pc_valobj.GetValueAsUnsigned(), 0x0800269E)

        xpsr_valobj = (
            process.GetThreadAtIndex(0).GetFrameAtIndex(0).FindRegister("xpsr")
        )
        self.assertEqual(xpsr_valobj.GetValueAsUnsigned(), 0x61000000)

        msp_valobj = process.GetThreadAtIndex(0).GetFrameAtIndex(0).FindRegister("msp")
        err = msp_valobj.GetError()
        self.assertTrue(err.Fail(), "lldb should not be able to fetch the msp register")

        val = b"\x11\x22\x33\x44"
        error = lldb.SBError()
        data = lldb.SBData()
        data.SetData(error, val, lldb.eByteOrderBig, 4)
        self.assertEqual(r1_valobj.SetData(data, error), True)
        self.assertSuccess(error)

        r1_valobj = process.GetThreadAtIndex(0).GetFrameAtIndex(0).FindRegister("r1")
        self.assertEqual(r1_valobj.GetValueAsUnsigned(), 0x11223344)



import lldb
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test_cpp_this(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// check this", lldb.SBFileSpec("main.cpp")
        )
        self.expect("frame variable m_field", startstr="(int) m_field = 30")



"""
Test that apropos env doesn't crash trying to touch the process plugin command
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class AproposWithProcessTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.cpp", "// break here")

    def test_apropos_with_process(self):
        """Test that apropos env doesn't crash trying to touch the process plugin command."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break in main() after the variables are assigned values.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        self.runCmd("apropos env")



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.GetFrameID()
    obj.GetPC()
    obj.SetPC(0xFFFFFFFF)
    obj.GetSP()
    obj.GetFP()
    obj.GetPCAddress()
    obj.GetSymbolContext(0)
    obj.GetModule()
    obj.GetCompileUnit()
    obj.GetFunction()
    obj.GetSymbol()
    obj.GetBlock()
    obj.GetFunctionName()
    obj.IsInlined()
    obj.EvaluateExpression("x + y")
    obj.EvaluateExpression("x + y", lldb.eDynamicCanRunTarget)
    obj.GetFrameBlock()
    obj.GetLineEntry()
    obj.GetThread()
    obj.Disassemble()
    obj.GetVariables(True, True, True, True)
    obj.GetVariables(True, True, True, False, lldb.eDynamicCanRunTarget)
    obj.GetRegisters()
    obj.FindVariable("my_var")
    obj.FindVariable("my_var", lldb.eDynamicCanRunTarget)
    obj.FindValue("your_var", lldb.eValueTypeVariableGlobal)
    obj.FindValue("your_var", lldb.eValueTypeVariableStatic, lldb.eDynamicCanRunTarget)
    obj.GetDescription(lldb.SBStream())
    obj.Clear()



# TestSwiftReplInC.py
#
# This source file is part of the Swift.org open source project
#
# Copyright (c) 2014 - 2018 Apple Inc. and the Swift project authors
# Licensed under Apache License v2.0 with Runtime Library Exception
#
# See https://swift.org/LICENSE.txt for license information
# See https://swift.org/CONTRIBUTORS.txt for the list of Swift project authors
#
# ------------------------------------------------------------------------------
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *
import lldbsuite.test.lldbutil as lldbutil

class TestSwiftReplInC(TestBase):

    NO_DEBUG_INFO_TESTCASE = True

    @swiftTest
    def test_repl_in_c(self):
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        self.expect("repl", error=True, substrs=["Swift standard library"])
        self.runCmd("kill")
        self.expect("repl", error=True, substrs=["running process"])



"""
Test calling a function that hits a signal set to auto-restart, make sure the call completes.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprCommandThatRestartsTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        self.main_source = "lotta-signals.c"
        self.main_source_spec = lldb.SBFileSpec(self.main_source)

    @skipIfDarwin  # llvm.org/pr19246: intermittent failure
    @skipIfWindows  # Test relies on signals, unsupported on Windows
    @expectedFlakeyAndroid(bugnumber="llvm.org/pr19246")
    @expectedFlakeyNetBSD
    def test(self):
        """Test calling function that hits a signal and restarts."""
        self.build()
        self.call_function()

    def check_after_call(self, num_sigchld):
        after_call = self.sigchld_no.GetValueAsSigned(-1)
        self.assertEqual(
            after_call - self.start_sigchld_no,
            num_sigchld,
            "Really got %d SIGCHLD signals through the call." % (num_sigchld),
        )
        self.start_sigchld_no = after_call

        # Check that we are back where we were before:
        frame = self.thread.GetFrameAtIndex(0)
        self.assertEqual(
            self.orig_frame_pc, frame.GetPC(), "Restored the zeroth frame correctly"
        )

    def call_function(self):
        (target, process, self.thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Stop here in main.", self.main_source_spec
        )

        # Make sure the SIGCHLD behavior is pass/no-stop/no-notify:
        self.runCmd("process handle SIGCHLD -s 0 -p 1 -n 0")

        # The sigchld_no variable should be 0 at this point.
        self.sigchld_no = target.FindFirstGlobalVariable("sigchld_no")
        self.assertTrue(self.sigchld_no.IsValid(), "Got a value for sigchld_no")

        self.start_sigchld_no = self.sigchld_no.GetValueAsSigned(-1)
        self.assertNotEqual(
            self.start_sigchld_no, -1, "Got an actual value for sigchld_no"
        )

        options = lldb.SBExpressionOptions()
        # processing 30 signals takes a while, increase the expression timeout
        # a bit
        options.SetTimeoutInMicroSeconds(3000000)  # 3s
        options.SetUnwindOnError(True)

        frame = self.thread.GetFrameAtIndex(0)
        # Store away the PC to check that the functions unwind to the right
        # place after calls
        self.orig_frame_pc = frame.GetPC()

        num_sigchld = 30
        value = frame.EvaluateExpression("call_me (%d)" % (num_sigchld), options)
        self.assertTrue(value.IsValid())
        self.assertSuccess(value.GetError())
        self.assertEqual(value.GetValueAsSigned(-1), num_sigchld)

        self.check_after_call(num_sigchld)

        # Okay, now try with a breakpoint in the called code in the case where
        # we are ignoring breakpoint hits.
        handler_bkpt = target.BreakpointCreateBySourceRegex(
            "Got sigchld %d.", self.main_source_spec
        )
        self.assertTrue(handler_bkpt.GetNumLocations() > 0)
        options.SetIgnoreBreakpoints(True)
        options.SetUnwindOnError(True)

        value = frame.EvaluateExpression("call_me (%d)" % (num_sigchld), options)

        self.assertTrue(value.IsValid())
        self.assertSuccess(value.GetError())
        self.assertEqual(value.GetValueAsSigned(-1), num_sigchld)
        self.check_after_call(num_sigchld)

        # Now set the signal to print but not stop and make sure that calling
        # still works:
        self.runCmd("process handle SIGCHLD -s 0 -p 1 -n 1")

        value = frame.EvaluateExpression("call_me (%d)" % (num_sigchld), options)

        self.assertTrue(value.IsValid())
        self.assertSuccess(value.GetError())
        self.assertEqual(value.GetValueAsSigned(-1), num_sigchld)
        self.check_after_call(num_sigchld)

        # Now set this unwind on error to false, and make sure that we still
        # complete the call:
        options.SetUnwindOnError(False)
        value = frame.EvaluateExpression("call_me (%d)" % (num_sigchld), options)

        self.assertTrue(value.IsValid())
        self.assertSuccess(value.GetError())
        self.assertEqual(value.GetValueAsSigned(-1), num_sigchld)
        self.check_after_call(num_sigchld)

        # Okay, now set UnwindOnError to true, and then make the signal behavior to stop
        # and see that now we do stop at the signal point:

        self.runCmd("process handle SIGCHLD -s 1 -p 1 -n 1")

        value = frame.EvaluateExpression("call_me (%d)" % (num_sigchld), options)
        self.assertTrue(value.IsValid())
        self.assertFalse(value.GetError().Success())

        # Set signal handling back to no-stop, and continue and we should end
        # up back in out starting frame:
        self.runCmd("process handle SIGCHLD -s 0 -p 1 -n 1")

        error = process.Continue()
        self.assertSuccess(error, "Continuing after stopping for signal succeeds.")

        frame = self.thread.GetFrameAtIndex(0)
        self.assertEqual(
            frame.GetPC(),
            self.orig_frame_pc,
            "Continuing returned to the place we started.",
        )
